@@ -128,6 +128,14 @@ int get_dispatch (int * qq)
     }
   fprintf(stderr, 
 	  "get_dispatch: did not find sys_call_table in system_call code\n");
+  fprintf(stderr, 
+	  "** This indicates that either your System.map does not match\n");
+  fprintf(stderr,
+	  "** the currently running kernel, or that your System.map does\n");
+  fprintf(stderr,
+	  "** not provide the required information, and thus use of\n");
+  fprintf(stderr,
+	  "** the --with-kcheck option is not possible\n");
   return -1;
 }
 
@@ -446,6 +454,19 @@ int main(int argc, char * argv[])
       fprintf(stderr, "kern_head: fill_smap failed\n");
       exit (EXIT_FAILURE);
     }
+  if (addr_system_call == 0L) 
+    {
+      printf("#endif\n");
+      fprintf(stderr, 
+	      "kern_head: address of system_call not found in System.map\n");
+      fprintf(stderr, 
+	      "** This indicates that your System.map does not provide\n");
+      fprintf(stderr, 
+	      "** the required information, and thus use of the\n");
+      fprintf(stderr, 
+	      "** --with-kcheck option is not possible\n");
+      exit (EXIT_FAILURE);
+    }
 
   for (i = 0; i < maxcall; ++i)
     {
@@ -692,9 +713,9 @@ int main(int argc, char * argv[])
   if (0 != getuid())
     {
       fprintf(stderr, "\n");
-      fprintf(stderr, "NOTE:  kern_head: must run as 'root'");
+      fprintf(stderr, "NOTE:  kern_head: must run as 'root' ");
       fprintf(stderr, "(need to read from /dev/kmem)\n");
-      fprintf(stderr, "       If you get this message, then proceed");
+      fprintf(stderr, "       If you get this message, then proceed ");
       fprintf(stderr, "as follows:\n");
       fprintf(stderr, "       $ su\n");
       fprintf(stderr, "       $ ./kern_head > sh_ks.h\n");
@@ -783,20 +804,21 @@ int main(int argc, char * argv[])
 	}
 
       if (verbose > 0)
-	fprintf(stderr, "(sysent) %#lx   %#lx (syscall)  %03d [%s]\n",
+	fprintf(stderr, "(kvm_nlist) %#lx   %#lx (sysent[%03d])  %03d [%s]\n",
 		(unsigned long) sys_list[i].n_value,
 		(unsigned long) sy.sy_call,
-		i, sys_list[i].n_name);
+		i, i, sys_list[i].n_name);
 
       if((unsigned long)sy.sy_call != sys_list[i].n_value && 
 	 sys_list[i].n_value != 0 &&
-	 0 != strcmp(sys_list[i].n_name, "_nosys"))  
+	 0 != strcmp(sys_list[i].n_name, "_nosys") &&
+	 (unsigned long)sy.sy_call != sys_list[290].n_value)  
 	{
           fprintf(stderr,
-                  "WARNING: (sysent) %#lx != %#lx (syscall)  %03d [%s]\n",
+                  "WARNING: (kvm_nlist) %#lx != %#lx (sysent[%03d])  %03d [%s]\n",
 		  (unsigned long) sys_list[i].n_value,
 		  (unsigned long) sy.sy_call,
-		  i, sys_list[i].n_name);
+		  i, i, sys_list[i].n_name);
 	}
       sh_smap[i].addr = (unsigned long) sy.sy_call;
       strncpy(sh_smap[i].name, sys_list[i].n_name, 64);
