@@ -115,6 +115,7 @@ volatile  int      sig_fresh_trail;        /* SIGIOT  */
 volatile  int      sig_config_read_again;  /* SIGHUP  */
 volatile  int      sig_terminate;          /* SIGQUIT */
 volatile  int      sig_termfast;           /* SIGTERM */
+volatile  int      sig_force_check;        /* SIGTTOU */
 long int           eintr__result;
 char               sh_sig_msg[SH_MINIBUF];
 
@@ -135,7 +136,7 @@ char               sh_sig_msg[SH_MINIBUF];
 #define GLOB_LEN 511
 #endif
 
-char * globber(char * str)
+char * globber(const char * str)
 {
   size_t i;
   size_t j;
@@ -259,6 +260,7 @@ void sh_init (void)
   sig_fresh_trail        = 0;           /* SIGIOT  */
   sig_terminate          = 0;           /* SIGQUIT */
   sig_termfast           = 0;           /* SIGTERM */
+  sig_force_check        = 0;           /* SIGTTOU */
   strcpy ( sh_sig_msg, _("None"));
 
 #ifdef MKB_01
@@ -764,18 +766,17 @@ static pid_t * procdirSamhain ()
 
 static int killprocSamhain (pid_t pid)
 {
+  int i;
+
   /* fprintf(stderr, "Killing %d\n", pid); */
   if (pid > 0 && 0 == kill (pid, SIGTERM))
     {
-      (void) retry_msleep(1, 0);
-      if (0 != kill (pid, 0) && errno == ESRCH)
-	return (0);
-      (void) retry_msleep(2, 0);
-      if (0 != kill (pid, 0) && errno == ESRCH)
-	return (0);
-      (void) retry_msleep(2, 0);
-      if (0 != kill (pid, 0) && errno == ESRCH)
-	return (0);
+      for (i = 0; i < 16; ++i)
+	{
+	  (void) retry_msleep(1, 0);
+	  if (0 != kill (pid, 0) && errno == ESRCH)
+	    return (0);
+	}
       
       (void) kill (pid, SIGKILL);
       return (0);
@@ -909,7 +910,7 @@ static int samhainctl(int ctl, int * argc, char * argv[])
 	_exit (1);
       default:
 	times = 0;
-	while (times < 10) {
+	while (times < 64) {
 	  respid = waitpid(pid, &status, WNOHANG|WUNTRACED);
 	  if ((pid_t)-1 == respid)
 	    {
@@ -1289,7 +1290,7 @@ int main(int argc, char * argv[])
 
   /* close all other files
    */
-  sh_unix_closeall(3, -1);
+  sh_unix_closeall(3, -1); /* after processing CL options */
 
 
   /* --- Get user info. ---
@@ -1399,7 +1400,7 @@ int main(int argc, char * argv[])
 	  /* fetch the file from server to get checksum
 	   */
 	  sh_hash_init ();
-	  sh_hash_hashdelete ();
+	  /* sh_hash_hashdelete (); */
 	}
       else
 	{
@@ -1590,10 +1591,19 @@ int main(int argc, char * argv[])
 	  if (sig_termfast == 1)  /* SIGTERM */
 	    {
 	      TPT((0, FIL__, __LINE__, _("msg=<Terminate.>\n")));
-	      strncpy (sh_sig_msg, _("SIGTERM"), 20);
+	      /* strncpy (sh_sig_msg, _("SIGTERM"), 20); */
 	      --sig_raised; --sig_urgent;
 	      aud_exit (FIL__, __LINE__, EXIT_SUCCESS);
 	    }
+
+	  if (sig_force_check == 1) /* SIGTTOU */
+	    {
+	      TPT((0, FIL__, __LINE__, _("msg=<Check run triggered.>\n")));
+	      flag_check_1 = 1;
+	      flag_check_2 = 1;
+	      sig_force_check = 0;
+	      --sig_raised; 
+	    }
 	  
 	  if (sig_config_read_again == 1) /* SIGHUP */
 	    {
@@ -1721,7 +1731,7 @@ int main(int argc, char * argv[])
 	  if (sig_terminate == 1)  /* SIGQUIT */
 	    {
 	      TPT((0, FIL__, __LINE__, _("msg=<Terminate.>\n")));
-	      strncpy (sh_sig_msg, _("SIGQUIT"), 20);
+	      strncpy (sh_sig_msg, _("Quit"), 20);
 	      --sig_raised; --sig_urgent;
 	      aud_exit (FIL__, __LINE__, EXIT_SUCCESS);
 	    }
@@ -1731,6 +1741,7 @@ int main(int argc, char * argv[])
 	      TPT((0, FIL__, __LINE__, _("msg=<Debug switch.>\n")));
 	      sh_error_dbg_switch();
 	      sig_debug_switch = 0;
+	      --sig_raised;
 	    }
 	  
 	  if (sig_suspend_switch == 1)  /* SIGUSR2 */
@@ -1820,7 +1831,7 @@ int main(int argc, char * argv[])
 	  if (flag_check_1 == 1)
 	    (void) sh_files_chk ();
 
-	  if (sig_urgent == 1)
+	  if (sig_urgent > 0)
 	    continue;
 
 	  /*
@@ -1832,14 +1843,14 @@ int main(int argc, char * argv[])
 	      sh_hash_unvisited (ShDFLevel[SH_ERR_T_FILE]);
 	    }
 
-	  if (sig_urgent == 1)
+	  if (sig_urgent > 0)
 	    continue;
 
 	  /* reset
 	   */
 	  TPT((0, FIL__, __LINE__, _("msg=<Reset status.>\n")))
 	  sh_dirs_reset  ();
-	  if (sig_urgent == 1)
+	  if (sig_urgent > 0)
 	    continue;
 
 	  sh_files_reset ();
@@ -1848,7 +1859,7 @@ int main(int argc, char * argv[])
 
 	  (void) sh_prelink_run (NULL, NULL, 0);
 
-	  if (sig_urgent == 1)
+	  if (sig_urgent > 0)
 	    continue;
 
 	  runtim = time(NULL) - sh.statistics.time_start;
@@ -1879,7 +1890,7 @@ int main(int argc, char * argv[])
 	    }
 	  sh.fileCheck.alarm_last = time (NULL);
 
-	  if (sig_urgent == 1)
+	  if (sig_urgent > 0)
 	    continue;
 
 	  /*
@@ -1891,7 +1902,7 @@ int main(int argc, char * argv[])
 #endif
 	}
       
-      if (sig_urgent == 1)
+      if (sig_urgent > 0)
 	continue;
       
       /* execute modules
@@ -1916,7 +1927,7 @@ int main(int argc, char * argv[])
        */
       if (sh.flag.isdaemon != S_TRUE && sh.flag.loop == S_FALSE)
 	break; 
-      if (sig_urgent == 1)
+      if (sig_urgent > 0)
 	continue;
       
       /* see whether its time to send mail
@@ -1929,7 +1940,7 @@ int main(int argc, char * argv[])
 	  sh.mailTime.alarm_last = time (NULL);
 	}
 #endif
-      if (sig_urgent == 1)
+      if (sig_urgent > 0)
 	continue;
             
       /* log the timestamp
@@ -1950,7 +1961,7 @@ int main(int argc, char * argv[])
        */
       (void) taus_seed();
       
-      if (sig_urgent == 1)
+      if (sig_urgent > 0)
 	continue;
       
       /* go to sleep
