@@ -227,13 +227,13 @@ typedef struct
         int  count;
 } md5Param;
 
-/* this typedef clashes with the Rijndael implementation (namespace ...) */
-#ifndef SH_ENCRYPT
-typedef unsigned char        byte;
-#endif
+
+
+typedef unsigned char        sh_byte;
+
 
 extern int md5Reset(register md5Param* p);
-extern int md5Update(md5Param* p, const byte* data, int size);
+extern int md5Update(md5Param* p, const sh_byte* data, int size);
 extern int md5Digest(md5Param* p, uint32* data);
 
 static char db_name[64]     = ""; 
@@ -290,7 +290,7 @@ int sh_database_set_password (char * str)
 
 /******************************************************************
  *
- *  Oracle and unixODBC stuff, completely untested
+ *  Oracle and unixODBC stuff, only Oracle tested untested
  *
  *  Based on the code in the snort output plugin (spo_database.c).
  *  Copyright/license statement in spo_database.c:
@@ -326,6 +326,11 @@ static    SQLHSTMT    u_statement;
 static    SQLINTEGER  u_col;
 static    SQLINTEGER  u_rows;
 
+void sh_database_reset()
+{
+  return;
+}
+
 static
 int sh_database_query (char  * query, /*@out@*/ long * id)
 {
@@ -486,10 +491,35 @@ static    OCIStmt   * o_statement;
 static    text        o_errormsg[512];
 static    sb4         o_errorcode;
 
+static  int  connected = 0;
+
+void sh_database_reset()
+{
+  if (connected == 1) 
+    {
+      OCILogoff(o_servicecontext, o_error);
+      OCIHandleFree((dvoid *) o_statement,      OCI_HTYPE_STMT);
+      OCIHandleFree((dvoid *) o_servicecontext, OCI_HTYPE_SVCCTX);
+      OCIHandleFree((dvoid *) o_error,          OCI_HTYPE_ERROR);
+    }
+  connected = 0;
+  return;
+}
+
+static char * sh_stripnl (char * str)
+{
+  size_t len = sl_strlen(str);
+  if (len > 0)
+    {
+      if (str[len-1] == '\n')
+	str[len-1] == '\0';
+    }
+  return str;
+}
+
 static
 int sh_database_query (char  * query, /*@out@*/ long * id)
 {
-  static  int  connected = 0;
   static  int  bad_init  = 0;
   int          result    = 0;
   char         row_query[128];
@@ -513,6 +543,7 @@ int sh_database_query (char  * query, /*@out@*/ long * id)
          OCIErrorGet(o_error, 1, NULL, &o_errorcode, \
                      o_errormsg, sizeof(o_errormsg), \
                      OCI_HTYPE_ERROR); \
+         sh_stripnl (o_errormsg); \
          sh_error_handle((-1), FIL__, __LINE__, 0, MSG_E_SUBGEN, \
 		     o_errormsg, _("sh_database_query")); \
          sl_snprintf(row_query, 127, \
@@ -585,12 +616,13 @@ int sh_database_query (char  * query, /*@out@*/ long * id)
   /* logon and allocate the service context handle 
    */
   if (OCILogon(o_environment, o_error, &o_servicecontext,
-	       db_user,     sl_strlen(db_user), 
-	       db_password, sl_strlen(db_password), 
-	       db_name,     sl_strlen(db_name))) 
+	       (OraText*) db_user,     sl_strlen(db_user), 
+	       (OraText*) db_password, sl_strlen(db_password), 
+	       (OraText*) db_name,     sl_strlen(db_name))) 
       {   
 	OCIErrorGet(o_error, 1, NULL, &o_errorcode, 
 		    o_errormsg, sizeof(o_errormsg), OCI_HTYPE_ERROR);
+	sh_stripnl (o_errormsg);
 	sh_error_handle((-1), FIL__, __LINE__, 0, MSG_E_SUBGEN,
 			o_errormsg, 
 			_("sh_database_query"));
@@ -632,12 +664,13 @@ int sh_database_query (char  * query, /*@out@*/ long * id)
 #endif
 
   if (OCIStmtPrepare(o_statement, o_error, 
-		     query, sl_strlen(query), 
+		     (OraText*) query, sl_strlen(query), 
 		     OCI_NTV_SYNTAX, OCI_DEFAULT))
     {
       OCIErrorGet(o_error, 1, NULL, 
 		  &o_errorcode, o_errormsg, sizeof(o_errormsg), 
 		  OCI_HTYPE_ERROR);
+      sh_stripnl (o_errormsg);
       sh_error_handle((-1), FIL__, __LINE__, 0, MSG_E_SUBGEN,
 		      o_errormsg, 
 		      _("sh_database_query"));
@@ -651,6 +684,7 @@ int sh_database_query (char  * query, /*@out@*/ long * id)
       OCIErrorGet(o_error, 1, NULL, 
 		  &o_errorcode, o_errormsg, sizeof(o_errormsg), 
 		  OCI_HTYPE_ERROR);
+      sh_stripnl (o_errormsg);
       sh_error_handle((-1), FIL__, __LINE__, 0, MSG_E_SUBGEN,
 		      o_errormsg, 
 		      _("sh_database_query"));
@@ -674,12 +708,14 @@ int sh_database_query (char  * query, /*@out@*/ long * id)
 		  _("sh_database_query"));
 #endif
 
-  if (OCIStmtPrepare(o_statement, o_error, row_query, sl_strlen(row_query), 
+  if (OCIStmtPrepare(o_statement, o_error, 
+		     (OraText*) row_query, sl_strlen(row_query), 
 		     OCI_NTV_SYNTAX, OCI_DEFAULT))
     {
       OCIErrorGet(o_error, 1, NULL, 
 		  &o_errorcode, o_errormsg, sizeof(o_errormsg), 
 		  OCI_HTYPE_ERROR);
+      sh_stripnl (o_errormsg);
       sh_error_handle((-1), FIL__, __LINE__, 0, MSG_E_SUBGEN,
 		      o_errormsg, 
 		      _("sh_database_query"));
@@ -692,6 +728,7 @@ int sh_database_query (char  * query, /*@out@*/ long * id)
       OCIErrorGet(o_error, 1, NULL, 
 		  &o_errorcode, o_errormsg, sizeof(o_errormsg), 
 		  OCI_HTYPE_ERROR);
+      sh_stripnl (o_errormsg);
       sh_error_handle((-1), FIL__, __LINE__, 0, MSG_E_SUBGEN,
 		      o_errormsg, 
 		      _("sh_database_query"));
@@ -705,6 +742,7 @@ int sh_database_query (char  * query, /*@out@*/ long * id)
       OCIErrorGet(o_error, 1, NULL, 
 		  &o_errorcode, o_errormsg, sizeof(o_errormsg), 
 		  OCI_HTYPE_ERROR);
+      sh_stripnl (o_errormsg);
       sh_error_handle((-1), FIL__, __LINE__, 0, MSG_E_SUBGEN,
 		      o_errormsg, 
 		      _("sh_database_query"));
@@ -715,6 +753,7 @@ int sh_database_query (char  * query, /*@out@*/ long * id)
       OCIErrorGet(o_error, 1, NULL, 
 		  &o_errorcode, o_errormsg, sizeof(o_errormsg), 
 		  OCI_HTYPE_ERROR);
+      sh_stripnl (o_errormsg);
       sh_error_handle((-1), FIL__, __LINE__, 0, MSG_E_SUBGEN,
 		      o_errormsg, 
 		      _("sh_database_query"));
@@ -771,6 +810,14 @@ int sh_database_query (char  * query, /*@out@*/ long * id)
 #include <libpq-fe.h>
 #endif
 
+static int        connection_status = S_FALSE;
+
+void sh_database_reset()
+{
+  connection_status = S_FALSE;
+  return;
+}
+
 static
 int sh_database_query (char  * query, /*@out@*/ long * id)
 {
@@ -779,7 +826,6 @@ int sh_database_query (char  * query, /*@out@*/ long * id)
   static PGconn   * conn = NULL;
   PGresult        * res;
   unsigned int      i;
-  static int        connection_status = S_FALSE;
   static SH_TIMEOUT sh_timer = { 0, 3600, S_TRUE };
 
   SL_ENTER(_("sh_database_query"));
@@ -915,13 +961,20 @@ int sh_database_query (char  * query, /*@out@*/ long * id)
 
 extern int flag_err_debug;
 
+static int        connection_status = S_FALSE;
+
+void sh_database_reset()
+{
+  connection_status = S_FALSE;
+  return;
+}
+
 static
 int sh_database_query (char  * query, /*@out@*/ long * id)
 {
   int               status = 0;
   char            * p;
   static MYSQL    * db_conn = NULL;
-  static int        connection_status = S_FALSE;
   static SH_TIMEOUT sh_timer = { 0, 3600, S_TRUE };
 
   SL_ENTER(_("sh_database_query"));
@@ -1165,13 +1218,17 @@ long sh_database_entry (dbins * db_entry, long id)
 
   /*@-type@*//* byte* versus char[..] */
   if (attr_tab[0].inHash == 1) 
-    (void) md5Update(&crc, db_entry->sev,  (int) strlen(db_entry->sev));
+    (void) md5Update(&crc, (sh_byte*) db_entry->sev,  
+		     (int) strlen(db_entry->sev));
   if (attr_tab[1].inHash == 1) 
-    (void) md5Update(&crc, db_entry->time, (int) strlen(db_entry->time));
+    (void) md5Update(&crc, (sh_byte*) db_entry->time, 
+		     (int) strlen(db_entry->time));
   if (attr_tab[2].inHash == 1) 
-    (void) md5Update(&crc, db_entry->host, (int) strlen(db_entry->host));
+    (void) md5Update(&crc, (sh_byte*) db_entry->host, 
+		     (int) strlen(db_entry->host));
   if (attr_tab[3].inHash == 1 && db_entry->msg[0] != '\0') 
-    (void) md5Update(&crc, db_entry->msg,  (int) strlen(db_entry->sev));
+    (void) md5Update(&crc, (sh_byte*) db_entry->msg,  
+		     (int) strlen(db_entry->sev));
   /*@+type@*/
 
   size   =  (int) (SH_QUERY_MAX - strlen(values));
@@ -1214,7 +1271,8 @@ long sh_database_entry (dbins * db_entry, long id)
 	  if (attr_tab[i].inHash == 1 && 
 	      ((char *)(db_entry)+attr_tab[i].off) != '\0')
 	    {
-	      (void)md5Update(&crc, ((char *)(db_entry)+attr_tab[i].off), 
+	      (void)md5Update(&crc, 
+			      (sh_byte*) ((char *)(db_entry)+attr_tab[i].off), 
 			      (int)strlen((char *)(db_entry)+attr_tab[i].off));
 	    }
 	  /*@+type@*/
@@ -1234,7 +1292,7 @@ long sh_database_entry (dbins * db_entry, long id)
 	      /*@-type@*//* byte* versus char[..] */
 	      (void)
 	      md5Update(&crc,
-			(char *) db_entry->long_data[attr_tab[i].val-START_SEC_LONGS], 
+			(sh_byte *) db_entry->long_data[attr_tab[i].val-START_SEC_LONGS], 
 			sizeof(long));
 	      /*@+type@*/
 	    }
