@@ -139,7 +139,7 @@ int sh_error_logverify (char * s)
   char c_start;
 #endif
   char signature[64];
-  char key[KEY_LEN+1];
+  char key[KEY_LEN+2];
   char path[KEY_LEN+1];
   char timestamp[64];
   char c_cont;
@@ -389,7 +389,12 @@ int sh_error_logverify (char * s)
 		    }
 		  (void) fflush(stdout); 
 		  key[0] = '\0';
-		  (void) fgets(key, KEY_LEN+1, stdin);
+		  (void) fgets(key, KEY_LEN+2, stdin);
+		  if (key[0] != '\n') 
+		    {
+		      if (key[strlen(key) - 1] == '\n')
+			key[strlen(key) - 1] = '\0';
+		    }
 		  if (key[0] == '/')
 		    {
 		      chk_mode = CHK_FIL;
