@@ -218,7 +218,6 @@ void sh_error_dbg_switch()
   return;
 }
 
-
 static int sh_error_set_classmask (/*@notnull@*/char * c, int * facility_mask)
 {
   char * p;
@@ -245,11 +244,11 @@ static int sh_error_set_classmask (/*@notnull@*/char * c, int * facility_mask)
   do {
     if (num == 0)
       {
-	p = strtok (c, " ,");
+	p = strtok (c, " ,\t");
 	++num;
       }
     else
-      p = strtok (NULL, " ,");
+      p = strtok (NULL, " ,\t");
 
     if (p == NULL)
       break;
@@ -684,16 +683,47 @@ int sh_error_set_database (char * str_s)
 int sh_error_set_prelude (char * str_s)
 {
   static int reject = 0;
+
   if (reject == 1)
     return (0);
 
   if (sh.flag.opts == S_TRUE)  
     reject = 1;
 
-  return (sh_error_set_level(str_s, &errFlags.preludelevel));
+  return sh_error_set_level(str_s, &errFlags.preludelevel);
 }
 #endif
 
+/* init or re-init log facilities that need it
+ */
+void sh_error_fixup()
+{
+#if defined(HAVE_LIBPRELUDE_9)
+  if ((errFlags.preludelevel & SH_ERR_NOT)   == 0)
+    sh_prelude_init();
+  else
+    sh_prelude_stop();
+#endif
+#ifdef WITH_DATABASE
+  sh_database_reset();
+#endif
+  return;
+}
+
+/* to be called from sh_prelude_reset
+ */
+void sh_error_init_prelude()
+{
+#if defined(HAVE_LIBPRELUDE_9)
+  if ((errFlags.preludelevel & SH_ERR_NOT)   == 0)
+    sh_prelude_init();
+  else
+    sh_prelude_stop();
+#endif
+  return;
+}
+
+
 /* set severity for mailing
  */
 int sh_error_setseverity (char * str_s)
@@ -1037,6 +1067,8 @@ void sh_error_handle (int sev, char * file, long line,
         {
           if (export_block == 0)
             {
+	      int retval;
+
 	      /* will truncate to 65280 bytes 
 	       */
               export_block = 1;
@@ -1045,9 +1077,14 @@ void sh_error_handle (int sev, char * file, long line,
 	      sprintf(ex_msg, _("%d?%u?%s"),             /* known to fit  */
 			   severity, class, lmsg->msg);
 	      /*@-bufferoverflowhigh@*/
-              (void) sh_forward (ex_msg);
+              retval = sh_forward (ex_msg);
 	      SH_FREE(ex_msg);
               export_block = 0;
+	      if (retval == -2)
+		{
+		  sh_error_handle ((-1), FIL__, __LINE__, 0, MSG_QUEUE_FULL,
+				   _("log server"));
+		}
             }
         }
 #endif
@@ -1064,17 +1101,24 @@ void sh_error_handle (int sev, char * file, long line,
 	{
 	  if (mail_block == 0)
 	    {
+	      int retval; 
+
 	      /* will truncate to 998 bytes 
 	       */
 	      mail_block = 1;
 
 	      BREAKEXIT(sh_mail_msg);
 	      if ( (severity & SH_ERR_FATAL) == 0) 
-		sh_mail_pushstack (lmsg->msg);
+		retval = sh_mail_pushstack (lmsg->msg);
 	      else 
-		(void) sh_mail_msg (lmsg->msg);
+		retval = sh_mail_msg (lmsg->msg);
 
 	      mail_block = 0;
+	      if (retval == -2)
+		{
+		  sh_error_handle ((-1), FIL__, __LINE__, 0, MSG_QUEUE_FULL,
+				   _("email"));
+		}
 	    }
 	}
 #endif
@@ -1099,7 +1143,8 @@ void sh_error_handle (int sev, char * file, long line,
 	      /*
 	       *  Reports first error after failure. Always tries.
 	       */
-	      (void) sh_prelude_alert (severity, (int) class, lmsg->msg);
+	      (void) sh_prelude_alert (severity, (int) class, lmsg->msg,
+				       lmsg->status, msg_id);
 
 	      prelude_block = 0;
 	    }
@@ -1232,10 +1277,10 @@ int sh_error_ehead (/*@null@*/char * str_s)
   SL_RETURN( 0, _("sh_error_ehead"));
 }
 
-#if !defined (VA_COPY)
-#if defined (__GNUC__) && defined (__PPC__) && (defined (_CALL_SYSV) || defined (_WIN32))
+#if !defined(VA_COPY)
+#if defined(__GNUC__) && defined(__PPC__) && (defined(_CALL_SYSV) || defined(_WIN32))
 #define VA_COPY(ap1, ap2)     (*(ap1) = *(ap2))
-#elif defined (VA_COPY_AS_ARRAY)
+#elif defined(VA_COPY_AS_ARRAY)
 #define VA_COPY(ap1, ap2)     memmove ((ap1), (ap2), sizeof (va_list))
 #else /* va_list is a pointer */
 #define VA_COPY(ap1, ap2)     ((ap1) = (ap2))
