@@ -282,7 +282,7 @@ int sh_ext_popen (sh_tas_t * task)
 	  /* don't leak file descriptors
 	   */
 #if !defined(PDGBFILE)
-	  sh_unix_closeall (3, task->com_fd);
+	  sh_unix_closeall (3, task->com_fd); /* in child process */
 #endif
 
 	  /* drop root privileges, if possible && requested
@@ -935,9 +935,9 @@ int sh_ext_add (char * argstring, int * ntok, char * stok[])
   do
     {
       if (i == 0)
-	p = strtok (argstring, ", ");
+	p = strtok (argstring, ", \t");
       else
-	p = strtok (NULL, ", ");
+	p = strtok (NULL, ", \t");
       if (p == NULL)
 	break;
 
