@@ -182,7 +182,7 @@ unsigned long sh_files_chk ()
   while (ptr) 
     {
 
-      if (sig_urgent == 1) {
+      if (sig_urgent > 0) {
 	SL_RETURN(fcount, _("sh_files_chk"));
       }
 
@@ -870,7 +870,7 @@ unsigned long sh_dirs_chk (int which)
 
   while (ptr) 
     {
-      if (sig_urgent == 1) {
+      if (sig_urgent > 0) {
 	SL_RETURN(dcount, _("sh_dirs_chk"));
       }
 
@@ -967,7 +967,7 @@ unsigned long sh_dirs_chk (int which)
 	  ptr->checked = S_TRUE;
 	}
 
-      if (sig_urgent == 1) {
+      if (sig_urgent > 0) {
 	SL_RETURN(dcount, _("sh_dirs_chk"));
       }
 
@@ -1360,7 +1360,7 @@ static int sh_files_checkdir (int iclass, int idepth, char * iname,
 
   SL_ENTER(_("sh_files_checkdir"));
 
-  if (sig_urgent == 1) {
+  if (sig_urgent > 0) {
     SL_RETURN((0), _("sh_files_checkdir"));
   }
 
@@ -1419,6 +1419,11 @@ static int sh_files_checkdir (int iclass, int idepth, char * iname,
 			    iname,
 			    &theFile, NULL, iclass);
 
+  if ((sig_termfast == 1) || (sig_terminate == 1)) 
+    {
+      SL_RETURN((0), _("sh_files_checkdir"));
+    }
+
   if (status == -1)
     {
       SH_FREE(tmpname); 
@@ -1518,13 +1523,54 @@ static int sh_files_checkdir (int iclass, int idepth, char * iname,
       sl_strlcat(tmpcat, "/",                   PATH_MAX);
     sl_strlcat(tmpcat, dirlist->sh_d_name,      PATH_MAX);
     
-    dst_ptr         = fileList;
     rdepth_next     = rdepth - 1;
     class_next      = class;
     file_class_next = class;
     checked_flag    = -1;
     cchecked_flag   = -1;
-    
+
+    /* Wed Aug 24 2005 compare against dirListOne, dirListTwo
+     * this fixes the problem that the directory special file
+     * is checked with the policy of the parent directory
+     */
+    dst_ptr         = dirListOne;
+    while (dst_ptr) 
+      {
+	if (0 == sl_strcmp(dst_ptr->name,  tmpcat))
+	  {
+	    /* Tue Aug  6 22:13:27 CEST 2002 introduce file_class_next
+	     * this fixes the problem that a policy for the directory
+	     * inode erroneously becomes a policy for the directory itself.
+	     */
+	    file_class_next    = dst_ptr->class;
+	    checked_flag       = dst_ptr->checked;
+	    cchecked_flag      = dst_ptr->childs_checked;
+	    break;
+	  }
+	dst_ptr = dst_ptr->next;
+      }
+
+    if (checked_flag == -1)
+      {
+	dst_ptr         = dirListTwo;
+	while (dst_ptr) 
+	  {
+	    if (0 == sl_strcmp(dst_ptr->name,  tmpcat))
+	      {
+		/* Tue Aug  6 22:13:27 CEST 2002 introduce file_class_next
+		 * this fixes the problem that a policy for the directory
+		 * inode erroneously becomes a policy for the directory itself.
+		 */
+		file_class_next    = dst_ptr->class;
+		checked_flag       = dst_ptr->checked;
+		cchecked_flag      = dst_ptr->childs_checked;
+		break;
+	      }
+	    dst_ptr = dst_ptr->next;
+	  }
+      }
+
+    dst_ptr         = fileList;
     while (dst_ptr) 
       {
 	if (0 == sl_strcmp(dst_ptr->name,  tmpcat))
@@ -1534,7 +1580,6 @@ static int sh_files_checkdir (int iclass, int idepth, char * iname,
 	     * inode erroneously becomes a policy for the directory itself.
 	     */
 	    file_class_next    = dst_ptr->class;
-	    /* class_next         = dst_ptr->class; */
 	    checked_flag       = dst_ptr->checked;
 	    cchecked_flag      = dst_ptr->childs_checked;
 	    break;
@@ -1553,6 +1598,8 @@ static int sh_files_checkdir (int iclass, int idepth, char * iname,
 	  {
 	    ++theDir.NumDirs;
 	  }
+	SH_FREE(tmpcat);
+	dirlist = dirlist->next;
 	continue;
       }
     
@@ -1857,18 +1904,18 @@ static ShFileType sh_files_filecheck (int class, char * dirName,
     {
       int  dummy;
       static int rsrc_init = 0;
-      static char rsrc[5];
+      static char rsrc[17];
       char testpath[PATH_MAX];
 
       if (rsrc_init == 0) {
-	sl_strlcpy(rsrc, _("rsrc"), 5);
+	sl_strlcpy(rsrc, _("..namedfork/rsrc"), 17);
 	rsrc_init = 1;
       }
       sl_strlcpy (testpath, fullpath, PATH_MAX);
       sl_strlcat (testpath,      "/", PATH_MAX);
       sl_strlcat (testpath,     rsrc, PATH_MAX);
 
-      if (sl_strlen(testpath) == (5 + sl_strlen(fullpath)))
+      if (sl_strlen(testpath) == (17 + sl_strlen(fullpath)))
 	{
 	  if (0 == sh_unix_file_stat (testpath))
 	    {
@@ -2008,6 +2055,7 @@ int sh_files_test_setup_int (dirstack_t * pdirL)
 int sh_files_test_double (dirstack_t * firstList, dirstack_t * secondList)
 {
   int          count;
+  int          retval = 0;
 
   dirstack_t * saveList = firstList;
   dirstack_t * tempList;
@@ -2020,26 +2068,37 @@ int sh_files_test_double (dirstack_t * firstList, dirstack_t * secondList)
       while (tempList)
 	{
 	  if (0 == sl_strcmp(firstList->name, tempList->name))
-	    ++count;
+	    {
+	      ++count;
+	    }
 	  tempList = tempList->next;
 	}
       tempList = secondList;
       while (tempList)
 	{
 	  if (0 == sl_strcmp(firstList->name, tempList->name))
-	    ++count;
+	    {
+	      ++count;
+	    }
 	  tempList = tempList->next;
 	}
-      if (count > 1)
-	sh_error_handle ((-1), FIL__, __LINE__, 0, MSG_FI_DOUBLE,
-			 firstList->name);
+      if (count > 1) 
+	{
+	  sh_error_handle ((-1), FIL__, __LINE__, 0, MSG_FI_DOUBLE,
+			   firstList->name);
+	  retval = 1;
+	}
       firstList = firstList->next;
     }
-  return 0;
+  return retval;
 }
+
+extern void     aud_exit   (char * file, int line, int fd);
       
 int sh_files_test_setup ()
 {
+  int retval = 0;
+
   /* Test for modifications allowed in ReadOnly directory
    */  
   sh_files_test_setup_int (dirListOne);
@@ -2047,9 +2106,17 @@ int sh_files_test_setup ()
 
   /* Test for files/dirz defined twice
    */  
-  sh_files_test_double (dirListOne, dirListTwo);
-  sh_files_test_double (dirListTwo, dirListOne);
-  sh_files_test_double (fileList,   NULL);
+  retval = sh_files_test_double (dirListOne, dirListTwo);
+  if (retval != 0)
+    aud_exit(FIL__, __LINE__, EXIT_FAILURE);
+
+  retval = sh_files_test_double (dirListTwo, dirListOne);
+  if (retval != 0)
+    aud_exit(FIL__, __LINE__, EXIT_FAILURE);
+
+  retval = sh_files_test_double (fileList,   NULL);
+  if (retval != 0)
+    aud_exit(FIL__, __LINE__, EXIT_FAILURE);
   return 0;
 }
 
