@@ -693,6 +693,43 @@ static void hashinsert (sh_file_t * s)
 }
 
 
+/******************************************************************
+ *
+ * Get a single line
+ *
+ ******************************************************************/
+static FILE * sh_fin_fd = NULL;
+
+static int sh_hash_getline (FILE * fd, char * line, int sizeofline)
+{
+  register int  n = 0;
+  char        * res;
+
+  if (sizeofline < 2) {
+    line[0] = '\0';
+    return 0;
+  }
+  res = fgets(line, sizeofline, fd);
+  if (res == NULL)
+    {
+      line[0] = '\0';
+      return -1;
+    }
+  n = strlen(line);
+  if (n > 1) {
+    --n;
+    line[n] = '\0';
+  }
+  return n;
+}
+
+static void sh_hash_getline_end ()
+{
+  fclose (sh_fin_fd);
+  sh_fin_fd = NULL;
+  return;
+}
+
 /******************************************************************
  *
  * ------- Check functions -------
@@ -707,28 +744,46 @@ static int IsInit = 0;
  * Fast forward to start of data
  *
  ******************************************************************/
-int sh_hash_setdataent (SL_TICKET fd, char * line, int size)
+int sh_hash_setdataent (SL_TICKET fd, char * line, int size, char * file)
 {
   long i;
+  extern int get_the_fd (SL_TICKET ticket);
 
   SL_ENTER(_("sh_hash_setdataent"));
 
   sl_rewind (fd);
 
+  if (sh_fin_fd != NULL)
+    {
+      fclose (sh_fin_fd);
+      sh_fin_fd = NULL;
+    }
+
+  sh_fin_fd = fdopen(get_the_fd(fd), "rb");
+  if (!sh_fin_fd)
+    {
+      dlog(1, FIL__, __LINE__, 
+	   _("The file signature database: %s is not readable.\n"),
+	   (NULL == file) ? _("(null)") : file);
+      sh_error_handle ((-1), FIL__, __LINE__, 0, MSG_P_NODATA,
+		       ( (NULL == file) ? _("(null)") : file)
+		       );
+      aud_exit (FIL__, __LINE__, EXIT_FAILURE);
+    }
+
   while (1) 
     {
-      i =  sh_unix_getline (fd, line, size-1);
+      i =  sh_hash_getline (sh_fin_fd, line, size-1);
       if (i < 0 ) 
 	{
 	  SH_FREE(line);
 	  dlog(1, FIL__, __LINE__, 
 	       _("The file signature database: %s does not\ncontain any data, or the start-of-file marker is missing (unlikely,\nunless modified by hand).\n"),
-	       (NULL == file_path('D', 'R'))? _("(null)"):file_path('D', 'R'));
+	       (NULL == file) ? _("(null)") : file);
 	       
 	  sh_error_handle ((-1), FIL__, __LINE__, 0, MSG_P_NODATA,
-			   ( (NULL == file_path('D', 'R')) ? _("(null)") :
-			     file_path('D', 'R'))
-			    );
+			   ( (NULL == file) ? _("(null)") : file)
+			   );
 	  aud_exit (FIL__, __LINE__, EXIT_FAILURE);
 	}
 
@@ -742,6 +797,41 @@ int sh_hash_setdataent (SL_TICKET fd, char * line, int size)
   SL_RETURN( 1, _("sh_hash_setdataent"));
 }
 
+static int sh_hash_setdataent_old (SL_TICKET fd, char * line, int size, 
+				   char * file)
+{
+  long i;
+
+  SL_ENTER(_("sh_hash_setdataent_old"));
+
+  sl_rewind (fd);
+
+  while (1) 
+    {
+      i =  sh_unix_getline (fd, line, size-1);
+      if (i < 0 ) 
+	{
+	  SH_FREE(line);
+	  dlog(1, FIL__, __LINE__, 
+	       _("The file signature database: %s does not\ncontain any data, or the start-of-file marker is missing (unlikely,\nunless modified by hand).\n"),
+	       (NULL == file) ? _("(null)") : file);
+	       
+	  sh_error_handle ((-1), FIL__, __LINE__, 0, MSG_P_NODATA,
+			   ( (NULL == file) ? _("(null)") : file)
+			   );
+	  aud_exit (FIL__, __LINE__, EXIT_FAILURE);
+	}
+
+#if defined(SH_STEALTH)
+      if (0 == sl_strncmp (line, N_("[SOF]"), 5)) 
+#else
+      if (0 == sl_strncmp (line, _("[SOF]"),  5)) 
+#endif
+	break;
+    }
+  SL_RETURN( 1, _("sh_hash_setdataent_old"));
+}
+
 /******************************************************************
  *
  * Read next record
@@ -758,12 +848,16 @@ sh_file_t *  sh_hash_getdataent (SL_TICKET fd, char * line, int size)
 
   SL_ENTER(_("sh_hash_getdataent"));
 
+  (void) fd;
+
   /* Read next record -- Part One 
    */
   p = SH_ALLOC(sizeof(sh_file_t));
 
-  i = sl_read(fd, &ft, sizeof(sh_filestore_t));
-  if ( SL_ISERROR(i) || i == 0) 
+  i = fread (&ft, sizeof(sh_filestore_t), 1, sh_fin_fd);
+  /* i = sl_read(fd, &ft, sizeof(sh_filestore_t)); */
+  /* if ( SL_ISERROR(i) || i == 0) */
+  if (i < 1)
     {
       SH_FREE(p);
       SL_RETURN( NULL, _("sh_hash_getdataent"));
@@ -798,7 +892,7 @@ sh_file_t *  sh_hash_getdataent (SL_TICKET fd, char * line, int size)
 
   /* Read next record -- Part Two -- Fullpath
    */
-  i =  sh_unix_getline (fd, line, size-1);
+  i =  sh_hash_getline (sh_fin_fd, line, size-1);
   if (i < 0 ) 
     {
       SH_FREE(line);
@@ -824,7 +918,7 @@ sh_file_t *  sh_hash_getdataent (SL_TICKET fd, char * line, int size)
 
   /* Read next record -- Part Three -- Linkpath
    */
-  i =  sh_unix_getline (fd, line, size-1);
+  i =  sh_hash_getline (sh_fin_fd, line, size-1);
   if (i < 0 ) 
     {
       SH_FREE(line);
@@ -899,6 +993,9 @@ void sh_hash_init ()
   char * line = NULL;
 
 #if defined(WITH_GPG) || defined(WITH_PGP)
+  extern int get_the_fd (SL_TICKET ticket);
+  FILE *   fin_cp;
+
   char * buf  = NULL;
   int    bufc;
   int    flag_pgp = S_FALSE;
@@ -936,7 +1033,7 @@ void sh_hash_init ()
 
       tiger_fd = fd;
       sl_strlcpy (sh.data.hash, 
-		  sh_tiger_hash (file_path('C', 'R'), 
+		  sh_tiger_hash (file_path('C', 'R'), /*irrelevant, TIGER_FD*/ 
 				 TIGER_FD, 0),
 		  KEY_LEN+1);
       sl_rewind (fd);
@@ -981,6 +1078,12 @@ void sh_hash_init ()
 
       } /* new 1.4.8 */
 
+  if (sig_termfast == 1)  /* SIGTERM */
+    {
+      TPT((0, FIL__, __LINE__, _("msg=<Terminate.>\n")));
+      --sig_raised; --sig_urgent;
+      aud_exit (FIL__, __LINE__, EXIT_SUCCESS);
+    }
 
 #if defined(WITH_GPG) || defined(WITH_PGP)
   /* new 1.4.8: also checked for server data */
@@ -989,16 +1092,33 @@ void sh_hash_init ()
    */
   fdTmp = open_tmp();
 
+  fin_cp = fdopen(get_the_fd(fd), "rb");
   buf = (char *) SH_ALLOC(8192);
-  while ( (bufc = sh_unix_getline (fd, buf, 8191)) > 0) 
+
+
+  /* while ( (bufc = sh_unix_getline (fd, buf, 8191)) > 0) */
+  while (NULL != fgets(buf, 8192, fin_cp))
     {
+      bufc = 0; 
+      while (bufc < 8192) { 
+	if (buf[bufc] == '\n') { ++bufc; break; }
+	++bufc;
+      }
+
+      if (sig_termfast == 1)  /* SIGTERM */
+	{
+	  TPT((0, FIL__, __LINE__, _("msg=<Terminate.>\n")));
+	  --sig_raised; --sig_urgent;
+	  aud_exit (FIL__, __LINE__, EXIT_SUCCESS);
+	}
+
       if (flag_pgp == S_FALSE &&
-	  (0 == sl_strcmp(buf, _("-----BEGIN PGP SIGNED MESSAGE-----"))||
-	   0 == sl_strcmp(buf, _("-----BEGIN PGP MESSAGE-----")))
+	  (0 == sl_strcmp(buf, _("-----BEGIN PGP SIGNED MESSAGE-----\n"))||
+	   0 == sl_strcmp(buf, _("-----BEGIN PGP MESSAGE-----\n")))
 	  )
 	{
 	  flag_pgp = S_TRUE;
-	  sl_write_line(fdTmp, buf, bufc);
+	  sl_write(fdTmp, buf, bufc);
 	  continue;
 	}
       
@@ -1013,7 +1133,7 @@ void sh_hash_init ()
 	  else if (0 == sl_strncmp(buf, _("Hash:"), 5) ||
 		   0 == sl_strncmp(buf, _("NotDashEscaped:"), 15))
 	    {
-	      sl_write_line(fdTmp, buf, bufc);
+	      sl_write(fdTmp, buf, bufc);
 	      continue;
 	    }
 	  else
@@ -1026,15 +1146,18 @@ void sh_hash_init ()
 	}
       else if (flag_pgp == S_TRUE)
 	{
-	  sl_write_line(fdTmp, buf, bufc);
+	  /* sl_write_line(fdTmp, buf, bufc); */
+	  sl_write(fdTmp, buf, bufc);
 	}
       
       if (flag_pgp == S_TRUE && 
-	  0 == sl_strcmp(buf, _("-----END PGP SIGNATURE-----")))
+	  0 == sl_strcmp(buf, _("-----END PGP SIGNATURE-----\n")))
 	break;
     }
   SH_FREE(buf);
   sl_close(fd);
+  fclose(fin_cp);
+
   fd = fdTmp;
   sl_rewind (fd);
 
@@ -1052,10 +1175,17 @@ void sh_hash_init ()
 
   /* fast forward to start of data
    */
-  sh_hash_setdataent(fd, line, MAX_PATH_STORE);
+  sh_hash_setdataent(fd, line, MAX_PATH_STORE, file_path('D', 'R'));
 
   while (1) 
     {
+      if (sig_termfast == 1)  /* SIGTERM */
+	{
+	  TPT((0, FIL__, __LINE__, _("msg=<Terminate.>\n")));
+	  --sig_raised; --sig_urgent;
+	  aud_exit (FIL__, __LINE__, EXIT_SUCCESS);
+	}
+
       p = sh_hash_getdataent (fd, line, MAX_PATH_STORE);
       if (p != NULL)
 	{
@@ -1072,6 +1202,7 @@ void sh_hash_init ()
   /* Always keep db in memory, so we have no open file
    */
   sl_close (fd);
+  sh_hash_getline_end();
   fd = -1;
 
   /* --- Initialization done successfully. ---
@@ -1175,6 +1306,9 @@ void sh_hash_pushdata (file_type * buf, char * fileHash)
   char   timestring[81];
   char * timep;
 
+#if !defined(__linux__)
+  int    i;
+#endif
 
   SL_ENTER(_("sh_hash_pushdata"));
 
@@ -1229,7 +1363,8 @@ void sh_hash_pushdata (file_type * buf, char * fileHash)
 
   if ((pushdata_stdout == S_FALSE) && (pushdata_isfirst == 1))  
     {
-      /* Warn that file already exists
+      /* Warn that file already exists; file_path != NULL here because
+       * checked above
        */
       if (0 == retry_lstat(FIL__, __LINE__, file_path('D', 'W'), &sbuf))
 	sh_error_handle((-1), FIL__, __LINE__, 0, MSG_FI_DBEX,
@@ -1265,7 +1400,9 @@ void sh_hash_pushdata (file_type * buf, char * fileHash)
 	    SL_RET0(_("sh_hash_pushdata"));
 	  }
 	  line = SH_ALLOC(MAX_PATH_STORE+1);
-	  if (SL_ISERROR(sh_hash_setdataent (pushdata_fd, line, MAX_PATH_STORE)))
+	  if (SL_ISERROR(sh_hash_setdataent_old (pushdata_fd, line, 
+						 MAX_PATH_STORE, 
+						 file_path('D', 'W'))))
 	    {
 	      SH_FREE(fullpath);
 	      SH_FREE(linkpath);
@@ -1361,6 +1498,9 @@ void sh_hash_pushdata (file_type * buf, char * fileHash)
     }
 #if defined(__linux__)
     sl_strlcpy(p.c_attributes, buf->c_attributes, 13);
+#else
+    for (i = 0; i < 12; ++i) p.c_attributes[i] = '-';
+    p.c_attributes[12] = '\0';
 #endif
     
 #if defined(SH_STEALTH)
@@ -1368,13 +1508,14 @@ void sh_hash_pushdata (file_type * buf, char * fileHash)
     sh_do_encode(p.c_owner,  sl_strlen(p.c_owner));
     sh_do_encode(p.c_group,  sl_strlen(p.c_group));
     sh_do_encode(p.checksum, sl_strlen(p.checksum));
-#if defined(__linux__)
+
     sh_do_encode(p.c_attributes,   sl_strlen(p.c_attributes));
 #endif
-#endif
     
 #if defined(__linux__)
     p.attributes  = (UINT32) buf->attributes;
+#else
+    p.attributes  = 0;
 #endif
     p.linkmode    = (UINT32) buf->linkmode;
     p.hardlinks   = (UINT32) buf->hardlinks;
@@ -1401,9 +1542,7 @@ void sh_hash_pushdata (file_type * buf, char * fileHash)
     swap_64(&(p.ctime));
     swap_32(&(p.owner));
     swap_32(&(p.group));
-#if defined(__linux__)
     swap_32(&(p.attributes));
-#endif
 
 #ifdef OLD_BUG
     swap_short(&(p.mark));
@@ -1614,6 +1753,139 @@ int sh_hash_set_visited_true (char * newname)
 }
 
 
+/******************************************************************
+ *
+ * Data entry for arbitrary data into database
+ *
+ ******************************************************************/
+
+static char * sh_hash_charhex( unsigned char i )
+{
+  static char i2h[2];
+  int j, k;
+
+  j = i / 16;
+  k = i - (j*16);
+
+  if (j < 10) i2h[0] = '0'+j;
+  else        i2h[0] = 'A'+(j-10);
+  
+  if (k < 10) i2h[1] = '0'+k;
+  else        i2h[1] = 'A'+(k-10);
+
+  return i2h;
+}
+
+void sh_hash_push2db (char * key, unsigned long val1, 
+		      unsigned long val2, unsigned long val3,
+		      unsigned char * str, int size)
+{
+  file_type   tmpFile;
+  int         i = 0;
+  char      * p;
+
+  sl_strlcpy(tmpFile.fullpath, key, PATH_MAX);
+  tmpFile.size  = val1;
+  tmpFile.mtime = val2;
+  tmpFile.ctime = val3;
+
+  tmpFile.atime = 0;
+  tmpFile.mode  = 0;
+  tmpFile.owner = 0;
+  tmpFile.group = 0;
+  sl_strlcpy(tmpFile.c_owner, _("root"), 5);
+  sl_strlcpy(tmpFile.c_group, _("root"), 5);
+
+  if ((str != NULL) && (size < (PATH_MAX/2)-1))
+    {
+      tmpFile.c_mode[0] = 'l';  
+      tmpFile.c_mode[1] = 'r'; tmpFile.c_mode[2]  = 'w';
+      tmpFile.c_mode[3] = 'x'; tmpFile.c_mode[4]  = 'r'; 
+      tmpFile.c_mode[5] = 'w'; tmpFile.c_mode[6]  = 'x'; 
+      tmpFile.c_mode[7] = 'r'; tmpFile.c_mode[8]  = 'w'; 
+      tmpFile.c_mode[9] = 'x'; tmpFile.c_mode[10] = '\0'; 
+      for (i = 0; i < size; ++i)
+	{
+	  p = sh_hash_charhex (str[i]);
+	  tmpFile.linkpath[2*i]   = p[0];
+	  tmpFile.linkpath[2*i+1] = p[1];
+	  tmpFile.linkpath[2*i+2] = '\0';
+	}
+    }
+  else
+    {
+      tmpFile.c_mode[0] = '-';  
+      tmpFile.c_mode[1] = '-'; tmpFile.c_mode[2]  = '-';
+      tmpFile.c_mode[3] = '-'; tmpFile.c_mode[4]  = '-'; 
+      tmpFile.c_mode[5] = '-'; tmpFile.c_mode[6]  = '-'; 
+      tmpFile.c_mode[7] = '-'; tmpFile.c_mode[8]  = '-'; 
+      tmpFile.c_mode[9] = '-'; tmpFile.c_mode[10] = '\0'; 
+      tmpFile.linkpath[0] = '-'; tmpFile.linkpath[1] = '\0';
+    }
+
+  sh_hash_pushdata (&tmpFile, 
+		    _("000000000000000000000000000000000000000000000000"));
+  return;
+}
+
+extern int sh_util_hextobinary (char * binary, char * hex, int bytes);
+
+char * sh_hash_db2pop (char * key, unsigned long * val1, 
+		       unsigned long * val2, unsigned long * val3,
+		       int * size)
+{
+  file_type   tmpFile;
+  size_t      len;
+  char      * p;
+  int         i;
+
+  *size = 0;
+
+  if (0 == sh_hash_get_it (key, &tmpFile))
+    {
+      *val1  = tmpFile.size;
+      *val2 = tmpFile.mtime;
+      *val3 = tmpFile.ctime;
+
+      if (tmpFile.linkpath[0] != '-')
+	{
+	  len = strlen(tmpFile.linkpath);
+
+	  p = SH_ALLOC((len/2)+1);
+	  i = sh_util_hextobinary (p, tmpFile.linkpath, len);
+
+	  if (i == 0)
+	    {
+	      *size = (len/2);
+	      p[*size] = '\0';
+	      return p;
+	    }
+	  else
+	    {
+	      SH_FREE(p);
+	      *size = 0;
+	      return NULL;
+	    }
+	}
+      else
+	{
+	  *size = 0;
+	  return NULL;
+	}
+    }
+  else
+    {
+      *size = -1;
+      *val1 =  0;
+      *val2 =  0;
+      *val3 =  0;
+      return NULL;
+    }
+}
+
+
+
+
 /******************************************************************
  *
  * Data entry in hash table
@@ -2008,7 +2280,7 @@ static char * all_items (file_type * theFile, char * fileHash, int is_new)
 /*****************************************************************
  *
  * Compare a file with the database status.
- *test_log
+ *
  *****************************************************************/
 int sh_hash_compdata (int class, file_type * theFile, char * fileHash,
 		      char * policy_override, int severity_override)
@@ -2275,7 +2547,12 @@ int sh_hash_compdata (int class, file_type * theFile, char * fileHash,
       msg = SH_ALLOC(SH_BUFSIZE);
       msg[0] = '\0';
 
-      if ((modi_mask & MODI_MOD) != 0)
+      if (   ((modi_mask & MODI_MOD) != 0)
+#if defined(HAVE_LIBPRELUDE) && defined(HAVE_LIBPRELUDE_9)
+	  || ((modi_mask & MODI_USR) != 0)
+	  || ((modi_mask & MODI_GRP) != 0)
+#endif
+	     )
 	{
 #if defined(__linux__)
 	  sl_snprintf(tmp, SH_BUFSIZE, 
@@ -2297,7 +2574,7 @@ int sh_hash_compdata (int class, file_type * theFile, char * fileHash,
 	  sl_snprintf(tmp, SH_BUFSIZE, 
 		      _("mode_old=\"%s\" mode_new=\"%s\" imode_old=\"%ld\" imode_new=\"%ld\" "),
 #else
-	  sl_snprintf(tmp, SH_BUFSIZE, _("mode_old=<%s>, mode_new=<%s>, "),
+		      sl_snprintf(tmp, SH_BUFSIZE, _("mode_old=<%s>, mode_new=<%s>, "),
 #endif
 		      p->theFile.c_mode, theFile->c_mode
 #ifdef SH_USE_XML
@@ -2307,17 +2584,20 @@ int sh_hash_compdata (int class, file_type * theFile, char * fileHash,
 #endif
 	  sl_strlcat(msg, tmp, SH_BUFSIZE);
 #ifdef REPLACE_OLD
-	  if (sh.flag.reportonce == S_TRUE && sh.flag.update == S_FALSE)
+	  if ((modi_mask & MODI_MOD) != 0)
 	    {
-	      sl_strlcpy(p->theFile.c_mode, theFile->c_mode, 11);
-	      p->theFile.mode = theFile->mode;
+	      if (sh.flag.reportonce == S_TRUE && sh.flag.update == S_FALSE)
+		{
+		  sl_strlcpy(p->theFile.c_mode, theFile->c_mode, 11);
+		  p->theFile.mode = theFile->mode;
 #if defined(__linux__)
-	      sl_strlcpy(p->theFile.c_attributes, theFile->c_attributes, 16);
-	      p->theFile.attributes = theFile->attributes;
+		  sl_strlcpy(p->theFile.c_attributes,theFile->c_attributes,16);
+		  p->theFile.attributes = theFile->attributes;
 #endif
+		}
 	    }
 #endif
-	  }
+	}
 
       if ((modi_mask & MODI_HLN) != 0)
 	{
@@ -2377,7 +2657,11 @@ int sh_hash_compdata (int class, file_type * theFile, char * fileHash,
 #endif
 	}
 
-      if ((modi_mask & MODI_USR) != 0)
+      if (   ((modi_mask & MODI_USR) != 0)
+#if defined(HAVE_LIBPRELUDE) && defined(HAVE_LIBPRELUDE_9)
+	  || ((modi_mask & MODI_MOD) != 0)
+#endif
+	  )
 	{
 #ifdef SH_USE_XML
 	  sl_snprintf(tmp, SH_BUFSIZE, _("owner_old=\"%s\" owner_new=\"%s\" iowner_old=\"%ld\" iowner_new=\"%ld\" "),
@@ -2391,15 +2675,21 @@ int sh_hash_compdata (int class, file_type * theFile, char * fileHash,
 		      );
 	  sl_strlcat(msg, tmp, SH_BUFSIZE); 
 #ifdef REPLACE_OLD
-	  if (sh.flag.reportonce == S_TRUE && sh.flag.update == S_FALSE)
-	    {
-	      sl_strlcpy(p->theFile.c_owner, theFile->c_owner, USER_MAX+2);
-	      p->theFile.owner = theFile->owner;
-	    }
+	  if ((modi_mask & MODI_USR) != 0) {
+	    if (sh.flag.reportonce == S_TRUE && sh.flag.update == S_FALSE)
+	      {
+		sl_strlcpy(p->theFile.c_owner, theFile->c_owner, USER_MAX+2);
+		p->theFile.owner = theFile->owner;
+	      }
+	  }
 #endif
 	}
 
-      if ((modi_mask & MODI_GRP) != 0)
+      if (   ((modi_mask & MODI_GRP) != 0)
+#if defined(HAVE_LIBPRELUDE) && defined(HAVE_LIBPRELUDE_9)
+	  || ((modi_mask & MODI_MOD) != 0)
+#endif
+	  )
 	{
 #ifdef SH_USE_XML
 	  sl_snprintf(tmp, SH_BUFSIZE, _("group_old=\"%s\" group_new=\"%s\" igroup_old=\"%ld\" igroup_new=\"%ld\" "),
@@ -2412,11 +2702,13 @@ int sh_hash_compdata (int class, file_type * theFile, char * fileHash,
 
 	  sl_strlcat(msg, tmp, SH_BUFSIZE); 
 #ifdef REPLACE_OLD
-	  if (sh.flag.reportonce == S_TRUE && sh.flag.update == S_FALSE)
-	    {
-	      sl_strlcpy(p->theFile.c_group, theFile->c_group, GROUP_MAX+2);
-	      p->theFile.group = theFile->group;
-	    }
+          if ((modi_mask & MODI_GRP) != 0) {
+	    if (sh.flag.reportonce == S_TRUE && sh.flag.update == S_FALSE)
+	      {
+		sl_strlcpy(p->theFile.c_group, theFile->c_group, GROUP_MAX+2);
+		p->theFile.group = theFile->group;
+	      }
+	  }
 #endif
 	}
 
@@ -2532,7 +2824,8 @@ int sh_hash_compdata (int class, file_type * theFile, char * fileHash,
 
 
       tmp_path = sh_util_safe_name(theFile->fullpath);
-      sh_error_handle(log_severity, FIL__, __LINE__, 0, MSG_FI_CHAN,
+      sh_error_handle(log_severity, FIL__, __LINE__, 
+		      (long) modi_mask, MSG_FI_CHAN,
 		      (policy_override == NULL) ? _(policy[class]):log_policy,
 		      change_code, tmp_path, msg);
 
@@ -2879,7 +3172,7 @@ int sh_hash_list_db (char * db_file)
 
   /* fast forward to start of data
    */
-  sh_hash_setdataent(fd, line, MAX_PATH_STORE);
+  sh_hash_setdataent(fd, line, MAX_PATH_STORE, db_file);
 
   while (1) 
     {
