@@ -1700,9 +1700,9 @@ int sh_kern_end ()
  * module timer
  *
  *************/
-int sh_kern_timer (unsigned long tcurrent)
+int sh_kern_timer (time_t tcurrent)
 {
-  if ((time_t) (tcurrent - lastcheck) >= ShKernInterval)
+  if ((int) (tcurrent - lastcheck) >= ShKernInterval)
     {
       lastcheck  = tcurrent;
       return (-1);
