@@ -404,14 +404,16 @@ void sh_mail_emptystack (void)
  */
 static char * split_string(char * str);
 
-void sh_mail_pushstack (/*@null@*/char * msg)
+int sh_mail_pushstack (/*@null@*/char * msg)
 {
   char * p;
+  int    retval = 0;
+  int    status;
 
   SL_ENTER(_("sh_mail_pushstack"));
 
   if (msg == NULL || failedMail == SL_TRUE || sh.srvmail.name[0] == '\0') 
-    SL_RET0(_("sh_mail_pushstack"));
+    SL_RETURN((0), (_("sh_mail_pushstack")));
 
 #if 0
   if (msg != NULL && sl_strlen(msg) > 998)  /* RFC 2822 */
@@ -426,7 +428,8 @@ void sh_mail_pushstack (/*@null@*/char * msg)
       fifo_init(fifo_mail);
     }
 
-  if (0 == push_list (fifo_mail, p))
+  status = push_list (fifo_mail, p);
+  if (status >= 0)
     ++sh.mailNum.alarm_last;
 
   SH_FREE(p);
@@ -434,10 +437,12 @@ void sh_mail_pushstack (/*@null@*/char * msg)
   if (sh.mailNum.alarm_last >= sh.mailNum.alarm_interval)
     {
       BREAKEXIT(sh_mail_msg);
-      (void) sh_mail_msg (NULL);
+      retval = sh_mail_msg (NULL);
     }
 
-  SL_RET0(_("sh_mail_pushstack"));
+  if (status == SH_FIFO_MAX)
+    retval = -2;
+  SL_RETURN(retval, (_("sh_mail_pushstack")));
 }
 
 
@@ -560,6 +565,7 @@ int sh_mail_msg (/*@null@*/char * message)
     size_t       wrlen;
     int          i;
     int          num_popped = 0;
+    int          retval = -1;  
 
     char       * bufcompress;
     static int   failcount = 0;
@@ -620,7 +626,7 @@ int sh_mail_msg (/*@null@*/char * message)
 
     /* Try at most each hour.
      */
-    if ((fail_time > 0) && (time(NULL) - fail_time) < 3600)
+    if ((fail_time > 0) && (time(NULL) - fail_time) < 3/*600*/)
       {
 	if (failcount > 3)
 	  {
@@ -919,17 +925,24 @@ int sh_mail_msg (/*@null@*/char * message)
       {
         while ( (NULL != (popMsg = pop_list(fifo_temp))) )
           {
-            if (0 == push_list (fifo_mail, popMsg))
+            if (push_list (fifo_mail, popMsg) >= 0)
 	      ++sh.mailNum.alarm_last;
             SH_FREE(popMsg);
           }
 	if (message != NULL)
 	  {
-#if 0
-	    push_list (fifo_mail, message);
-#endif
-	    if (0 == push_list (fifo_mail,  theMsg))
+	    if (fifo_mail == NULL)
+	      {
+		fifo_mail = SH_ALLOC(sizeof(SH_FIFO));
+		fifo_init(fifo_mail);
+	      }
+	    retval = push_list (fifo_mail,  theMsg);
+	    if (retval >= 0) 
 	      ++sh.mailNum.alarm_last;
+	    if (retval == SH_FIFO_MAX)
+	      retval = -2;
+	    else
+	      retval = -1;
 	  }
       }
     else if (fifo_temp != NULL)
@@ -957,7 +970,7 @@ int sh_mail_msg (/*@null@*/char * message)
     if (errcount == address_num)
       {
 	fail_time = time(NULL);
-	SL_RETURN((-1), _("sh_mail_msg"));
+	SL_RETURN((retval), _("sh_mail_msg"));
       }
     success_time = time(NULL);
     failcount = 0;
@@ -1026,6 +1039,26 @@ int sh_mail_set_relay (char * str_s)
   SL_RETURN( 0, _("sh_mail_set_relay"));
 }
 
+static char * mail_sender = NULL;
+
+int sh_mail_set_sender (char *str)
+{
+  if (mail_sender != NULL) 
+    {
+      SH_FREE (mail_sender);
+      mail_sender = NULL;
+    }
+  if (str != NULL)
+    {
+      mail_sender = sh_util_strdup (str);
+    }
+  if (mail_sender == NULL)
+    {
+      return -1;
+    }
+  return 0;
+}
+
 
 /*************************
  *
@@ -1053,7 +1086,7 @@ static FILE * sh_mail_start_conn (int aFlag)
   time_t       my_time;
   char         my_tbuf[128];
 
-  int                  fd;
+  int          fd;
 
   dnsrep     * answers;
   mx         * result;
@@ -1207,11 +1240,15 @@ static FILE * sh_mail_start_conn (int aFlag)
 
   /* tell them who we are
    */
-  (void) sl_strlcpy (this_address, DEFAULT_SENDER, 256);
+  (void) sl_strlcpy (this_address, 
+		     mail_sender ? mail_sender : DEFAULT_SENDER, 256);
   if (NULL == strchr(this_address, '@'))
     {
       (void) sl_strlcat (this_address, "@", 256);
-      (void) sl_strlcat (this_address, sh.host.name, 256);
+      if (0 != is_numeric(sh.host.name))
+	(void) sl_strlcat (this_address, _("example.com"), 256);
+      else
+	(void) sl_strlcat (this_address, sh.host.name, 256);
     }
 
   TPT(( 0, FIL__, __LINE__,  _("msg=<MAIL FROM:<%s>>%c%c"), 
@@ -1590,6 +1627,8 @@ static char * split_string(char * str)
  *
  *****************************************************************/
 
+#if defined(HAVE_ARPA_NAMESER_H)
+
 #include <netinet/in.h>
 #ifdef __APPLE__
 #define BIND_8_COMPAT 1
@@ -1824,7 +1863,12 @@ static dnsrep * get_mx (char *hostname)
   SL_RETURN (retval, _("get_mx"));
 }
 /* ifndef S_SPLINT_S */
-#endif 
+#endif
+
+/* #if defined(HAVE_ARPA_NAMESER_H) */
+#endif
+
+
 static int comp_mx_pref (const void * a, const void * b)
 {
   const mx * ax = (const mx *) a;
@@ -1851,8 +1895,10 @@ static dnsrep * return_mx (char *domain)
 
   SL_ENTER(_("return_mx"));
 
+#if defined(HAVE_ARPA_NAMESER_H)
   if (domain != NULL)
     answers = /*@-unrecog@*/get_mx (domain)/*@+unrecog@*/;
+#endif
 
   if (answers != NULL && answers->count > 0)
     {
@@ -1862,9 +1908,10 @@ static dnsrep * return_mx (char *domain)
     }
   else
     {
-#ifdef FIL__
       if (domain != NULL)
 	{
+#if defined(HAVE_ARPA_NAMESER_H)
+#ifdef FIL__
 	  (void) sl_strlcpy (errmsg, _("No MX record for domain "), 127);
 	  (void) sl_strlcat (errmsg, domain, 127);
 	  sh_error_handle (SH_ERR_ALL, FIL__, __LINE__, 0, MSG_E_SUBGEN,
@@ -1876,6 +1923,7 @@ static dnsrep * return_mx (char *domain)
 	  strncat (errmsg, domain, 100);
 	  errmsg[122] = '\0';
 	  fprintf(stderr, "Warning: %s\n", errmsg);
+#endif
 #endif
 	}
       if (domain != NULL)
