@@ -47,6 +47,10 @@
 #include "sh_database.h"
 #endif
 
+#ifdef HAVE_LIBPRELUDE_9
+#include "sh_prelude.h"
+#endif
+
 extern int set_reverse_lookup (char * c);
 
 #undef  FIL__
@@ -568,6 +572,10 @@ int sh_readconf_read (void)
 
   sl_close (fd);
 
+  sh_error_fixup();
+
+  read_mode = SH_SECTION_NONE; /* reset b/o sighup reload */
+
   SL_RETURN( 0, _("sh_readconf_read"));
 }
 
@@ -921,6 +929,10 @@ cfg_options ext_table[] = {
   sh_forward_lookup_level },
   { N_("setclientfromaccept"), SH_SECTION_SRV,  SH_SECTION_MISC, 
     set_socket_peer },
+  { N_("setserverport"),       SH_SECTION_SRV,  SH_SECTION_MISC, 
+    sh_forward_set_port },
+  { N_("setserverinterface"),  SH_SECTION_SRV,  SH_SECTION_MISC, 
+    sh_forward_set_interface },
   { N_("client"),              SH_SECTION_CLIENTS,           SH_SECTION_NONE, 
     sh_forward_register_client },
 #endif
@@ -955,6 +967,8 @@ cfg_options ext_table[] = {
     sh_mail_setFlag },
   { N_("mailsubject"),       SH_SECTION_MAIL,  SH_SECTION_MISC, 
     set_mail_subject },
+  { N_("setmailsender"),     SH_SECTION_MAIL,  SH_SECTION_MISC, 
+    sh_mail_set_sender },
 #endif
   { N_("setbindaddress"),    SH_SECTION_MISC,  SH_SECTION_NONE,
     sh_calls_set_bind_addr },
@@ -986,6 +1000,18 @@ cfg_options ext_table[] = {
     sh_error_set_prelude },
   { N_("preludeclass"),      SH_SECTION_LOG,   SH_SECTION_NONE, 
     sh_error_prelude_mask },
+#ifdef HAVE_LIBPRELUDE_9
+  { N_("preludeprofile"),    SH_SECTION_MISC,  SH_SECTION_NONE,
+    sh_prelude_set_profile },
+  { N_("preludemaptoinfo"),    SH_SECTION_MISC,  SH_SECTION_NONE,
+    sh_prelude_map_info },
+  { N_("preludemaptolow"),     SH_SECTION_MISC,  SH_SECTION_NONE,
+    sh_prelude_map_low },
+  { N_("preludemaptomedium"),  SH_SECTION_MISC,  SH_SECTION_NONE,
+    sh_prelude_map_medium },
+  { N_("preludemaptohigh"),    SH_SECTION_MISC,  SH_SECTION_NONE,
+    sh_prelude_map_high },
+#endif
 #endif
 
   { N_("logcalls"),          SH_SECTION_LOG,   SH_SECTION_NONE, 
