@@ -262,7 +262,7 @@ static int do_truncate_int (char * path, int depth)
 			   sh_error_message(errno));
 	  return -1;
 	} 
-      if (0 != fstat(fd, &two))
+      if (0 != retry_fstat(FIL__, __LINE__, fd, &two))
 	{ 
 	  sh_error_handle ((-1), FIL__, __LINE__, errno,
 			   MSG_SUID_ERROR,
@@ -426,7 +426,7 @@ int sh_suidchk_check_internal (char * iname)
       SL_RETURN( (-1), _("sh_suidchk_check_internal"));
     }
 
-  if (sig_urgent == 1) {
+  if (sig_urgent > 0) {
     SL_RETURN( (0), _("sh_suidchk_check_internal"));
   }
 
@@ -436,16 +436,16 @@ int sh_suidchk_check_internal (char * iname)
     {
       status = errno;
       tmp = sh_util_safe_name(iname);
-      sh_error_handle ((-1), FIL__, __LINE__, status, MSG_E_SUBGEN,
-		       sh_error_message(status),
-		       tmp );
+      sh_error_handle (ShDFLevel[SH_ERR_T_DIR], FIL__, __LINE__, status, 
+		       MSG_E_OPENDIR,
+		       sh_error_message (status), tmp);
       SH_FREE(tmp);
       SL_RETURN( (-1), _("sh_suidchk_check_internal"));
     }
 
   do {
 
-    if (sig_urgent == 1) {
+    if (sig_urgent > 0) {
       SL_RETURN( (0), _("sh_suidchk_check_internal"));
     }
 
@@ -664,7 +664,7 @@ int sh_suidchk_check_internal (char * iname)
 				      }
 				    break;
 				  case SH_Q_CHANGEPERM:
-				    if (stat(tmp, &fileInfo) == -1)
+				    if (retry_lstat(FIL__, __LINE__, tmpcat, &fileInfo) == -1)
 				      {
 					status = errno;
 					msg = SH_ALLOC(SH_BUFSIZE);
@@ -690,7 +690,7 @@ int sh_suidchk_check_internal (char * iname)
 					  fileInfo.st_mode -= S_ISUID;
 					if ((fileInfo.st_mode & S_ISGID) > 0)
 					  fileInfo.st_mode -= S_ISGID;
-					if (chmod(tmp, fileInfo.st_mode) == -1)
+					if (chmod(tmpcat, fileInfo.st_mode) == -1)
 					  {
 					    status = errno;
 					    msg = SH_ALLOC(SH_BUFSIZE);
@@ -737,7 +737,7 @@ int sh_suidchk_check_internal (char * iname)
 				      }
 				    else
 				      {
-					if (retry_stat (FIL__, __LINE__, 
+					if (retry_lstat (FIL__, __LINE__, 
 							theFile.fullpath, &fileInfo) == -1)
 					  {
 					    status = errno;
@@ -960,7 +960,7 @@ int sh_suidchk_check ()
 
   SL_ENTER(_("sh_suidchk_check"));
 
-  sh_error_handle (SH_ERR_INFO, FIL__, __LINE__, EINVAL, MSG_E_SUBGEN,
+  sh_error_handle (SH_ERR_NOTICE, FIL__, __LINE__, EINVAL, MSG_E_SUBGEN,
 		   _("Checking for SUID programs"),
 		   _("suidchk_check") );
 
