@@ -24,8 +24,8 @@
 #include "sh_error.h"
 #include "sh_utils.h"
 
-#ifdef HAVE_LONG_LONG
-typedef unsigned long long int word64;
+#ifdef HAVE_LONG_64
+typedef unsigned long int word64;
 #endif
 
 #if defined(HAVE_INT_32)
@@ -49,7 +49,7 @@ typedef unsigned char sh_byte;
 #undef  FIL__
 #define FIL__  _("sh_tiger0.c")
 
-#ifdef HAVE_LONG_LONG
+#ifdef HAVE_LONG_64
 void tiger_t(word64 *str, word64 length, word64 * res);
 void tiger(word64 *str, word64 length, word64 * res);
 
@@ -90,7 +90,7 @@ void tiger_dbg(sh_word32 res[6], int step,
 SL_TICKET tiger_fd = (-1);
 
 
-#ifdef HAVE_LONG_LONG
+#ifdef HAVE_LONG_64
 static
 word64 * sh_tiger_hash_val (char * filename, TigerType what, 
 			    unsigned long Length, int timeout)
@@ -121,7 +121,7 @@ sh_word32 * sh_tiger_hash_val (char * filename, TigerType what,
   /*@+nestedextern@*/
 #endif
 
-#ifdef HAVE_LONG_LONG
+#ifdef HAVE_LONG_64
 #define TIGER_CAST (word64*)
   static word64 res[3];
   res[0]= (word64) 0x0123456789ABCDEFLL;
@@ -193,6 +193,9 @@ sh_word32 * sh_tiger_hash_val (char * filename, TigerType what,
 
 	if (SL_ISERROR (count)) 
 	  {
+	    if (sig_termfast == 1) {
+	      SL_RETURN( NULL, _("sh_tiger_hash_val"));
+	    }
 	    TPT((0, FIL__ , __LINE__ , _("msg=<SL_ISERROR (%ld)>\n"), count)); 
 	    tmp = sh_util_safe_name (filename);
 	    if (count == SL_TIMEOUT)
@@ -357,10 +360,10 @@ sh_word32 * sh_tiger_hash_val (char * filename, TigerType what,
 
 typedef sh_word32     uint32;
 typedef unsigned char uint8;
-/* this typedef clashes with the Rijndael implementation (namespace ...) */
-#ifndef SH_ENCRYPT
-typedef unsigned char        byte;
-#endif
+
+
+
+
 
 /* Structure to save state of computation between the single steps.  */
 typedef struct
@@ -434,7 +437,7 @@ void md5Process(md5Param* p)
         register uint32 a,b,c,d;
         register uint32* w;
         #if WORDS_BIGENDIAN
-        register byte t;
+        register sh_byte t;
         #endif
 
         w = p->data;
@@ -526,7 +529,7 @@ void md5Process(md5Param* p)
         p->h[3] += d;
 }
 
-int md5Update(md5Param* p, const byte* data, int size)
+int md5Update(md5Param* p, const sh_byte* data, int size)
 {
         register int proclength;
 
@@ -534,7 +537,7 @@ int md5Update(md5Param* p, const byte* data, int size)
         {
 	  proclength = (((int)p->offset + size) > 64) ? 
 	    (64 - (int)p->offset) : size;
-	  memcpy(((byte *) p->data) + p->offset, data, (size_t) proclength);
+	  memcpy(((sh_byte *) p->data) + p->offset, data, (size_t) proclength);
 	  size -= proclength;
 	  data += proclength;
 	  p->offset += proclength;
@@ -681,6 +684,8 @@ static int md5_stream(char * filename, void *resblock, int timeout)
 
       if (SL_ISERROR (n))
 	{
+	  if (sig_termfast == 1)
+	    return -1;
 	  TPT((0, FIL__ , __LINE__ , _("msg=<SL_ISERROR (%ld)>\n"), n));
 	  tmp = sh_util_safe_name (filename);
 	  if (n == SL_TIMEOUT) 
@@ -711,7 +716,7 @@ static int md5_stream(char * filename, void *resblock, int timeout)
     /* Process buffer with BLOCKSIZE bytes.  Note that
        BLOCKSIZE % 64 == 0
     */
-    (void) md5Update(&ctx, (byte*) buffer, BLOCKSIZE);
+    (void) md5Update(&ctx, (sh_byte*) buffer, BLOCKSIZE);
     sh.statistics.bytes_hashed += BLOCKSIZE;
 
 #if defined (SH_WITH_CLIENT) || defined (SH_STANDALONE) 
@@ -730,7 +735,7 @@ static int md5_stream(char * filename, void *resblock, int timeout)
   /* Add the last bytes if necessary.  */
   if (sum > 0)
     {
-      (void) md5Update(&ctx, (byte*) buffer, (int) sum);
+      (void) md5Update(&ctx, (sh_byte*) buffer, (int) sum);
       sh.statistics.bytes_hashed += BLOCKSIZE;
     }
 
@@ -1188,6 +1193,9 @@ static int sha1_stream(char * filename, void *resblock, int timeout)
 
       if (SL_ISERROR (n))
 	{
+	  if (sig_termfast == 1)
+	    return -1;
+
 	  TPT((0, FIL__ , __LINE__ , _("msg=<SL_ISERROR (%ld)>\n"), n));
 	  tmp = sh_util_safe_name (filename);
 	  if (n == SL_TIMEOUT)
@@ -1197,9 +1205,11 @@ static int sha1_stream(char * filename, void *resblock, int timeout)
 				 timeout, tmp);
 	      }
 	    }
-	  else
-	    sh_error_handle (ShDFLevel[SH_ERR_T_FILE], FIL__, __LINE__, 0,
-			     MSG_E_READ, tmp);
+	  else 
+	    {
+	      sh_error_handle (ShDFLevel[SH_ERR_T_FILE], FIL__, __LINE__, 0,
+			       MSG_E_READ, tmp);
+	    }
 	  SH_FREE(tmp);
 	  return -1;
 	}
@@ -1284,7 +1294,7 @@ static char * sh_tiger_sha1_hash  (char * filename, TigerType what,
 
 static int hash_type = 0;
 
-int sh_tiger_gethashtype ()
+int sh_tiger_get_hashtype ()
 {
   return hash_type;
 }
@@ -1344,7 +1354,7 @@ char * sh_tiger_generic_hash (char * filename, TigerType what,
 static char * sh_tiger_hash_internal (char * filename, TigerType what, 
 				      unsigned long Length, int timeout)
 {
-#ifdef HAVE_LONG_LONG
+#ifdef HAVE_LONG_64
   word64 * res;
 #else
   sh_word32 * res;
@@ -1358,7 +1368,7 @@ static char * sh_tiger_hash_internal (char * filename, TigerType what,
   if (res != NULL)
     {
       /*@-bufferoverflowhigh -formatconst@*/
-#ifdef HAVE_LONG_LONG
+#ifdef HAVE_LONG_64
       sprintf(out,                                   /* known to fit  */
 	      MYFORMAT,
 	      (sh_word32)(res[0]>>32), 
@@ -1393,7 +1403,7 @@ char * sh_tiger_hash_gpg (char * filename, TigerType what,
   size_t  len;
   char * out;
   char   outhash[48+6+1];
-#ifdef HAVE_LONG_LONG
+#ifdef HAVE_LONG_64
   word64 * res;
 #else
   sh_word32 * res;
@@ -1405,7 +1415,7 @@ char * sh_tiger_hash_gpg (char * filename, TigerType what,
   if (res != NULL)
     {
       /*@-bufferoverflowhigh -formatconst@*/
-#ifdef HAVE_LONG_LONG
+#ifdef HAVE_LONG_64
       sprintf(outhash,                               /* known to fit  */
 	      GPGFORMAT,
 	      (sh_word32)(res[0]>>32), 
@@ -1460,7 +1470,7 @@ UINT32 * sh_tiger_hash_uint32 (char * filename,
 			       TigerType what, 
 			       unsigned long Length)
 {
-#ifdef HAVE_LONG_LONG
+#ifdef HAVE_LONG_64
   word64 * res;
 #else
   sh_word32 * res;
@@ -1476,7 +1486,7 @@ UINT32 * sh_tiger_hash_uint32 (char * filename,
 
   if (res != NULL)
     {
-#ifdef HAVE_LONG_LONG
+#ifdef HAVE_LONG_64
 	out[0] =  (UINT32)(res[0]>>32); 
 	out[1] =  (UINT32)(res[0]);
 	out[2] =  (UINT32)(res[1]>>32); 
