@@ -22,7 +22,7 @@
 #include "config_xor.h"
 
 
-#ifndef HAVE_LONG_LONG
+#ifndef HAVE_LONG_64
 
 
 /* sboxes32.c: Tiger S boxes for 32-bit-only compilers */
