@@ -51,12 +51,12 @@
 #endif
 
 #include <sys/socket.h>
-#include <netinet/in.h>
-#include <arpa/inet.h>
 
 #ifdef  HOST_IS_HPUX
 #define _XOPEN_SOURCE_EXTENDED
 #endif
+#include <netinet/in.h>
+#include <arpa/inet.h>
 #include <netdb.h>
 
 #ifndef FD_SET
@@ -351,17 +351,21 @@ int get_open_max ()
 {
   int value;
 
+#ifdef _SC_OPEN_MAX
+  value = sysconf (_SC_OPEN_MAX);
+#else
 #ifdef OPEN_MAX
   value = OPEN_MAX;
 #else
-  value = sysconf (_SC_OPEN_MAX);
+  value = _POSIX_OPEN_MAX;
+#endif
 #endif
 
   if (value < 0)
     value = 8;  /* POSIX lower limit */
 
-  if (value > 2048)
-    value = 2048;
+  if (value > 4096)
+    value = 4096;
 
   return value;
 }
@@ -1588,9 +1592,11 @@ SL_TICKET open_tmp ()
 
   } while (status == BAD);
 
-  fd = sl_open_rdwr (file, SL_YESPRIV);
+  fd = sl_open_safe_rdwr (file, SL_YESPRIV);
   if (SL_ISERROR(fd))
     {
+      sh_error_handle(SH_ERR_ALL, FIL__, __LINE__, fd, MSG_E_SUBGEN, 
+		      _("Error opening temporary file"), _("open_tmp"));
       TPT(( 0, FIL__, __LINE__, _("msg=<Error %d temporary file %s>\n"), 
 	    fd, file));
     }
