@@ -232,7 +232,7 @@ char * sh_unix_siglist (int signum)
 #endif
 #ifdef SIGTSTP
     case SIGTSTP: 
-      return _("Stopped");
+      return _("Stop typed at tty");
 #endif
 #ifdef SIGTTIN
     case SIGTTIN: 
@@ -580,6 +580,10 @@ void sh_unix_sigaction (int mysignal)
   if (mysignal == SIGHUP)
     sig_config_read_again = 1;
 #endif
+#ifdef SIGTTOU
+  if (mysignal == SIGTTOU)
+    sig_force_check = 1;
+#endif
 #ifdef SIGABRT
   if (mysignal == SIGABRT)
     sig_fresh_trail       = 1;
@@ -594,6 +598,7 @@ void sh_unix_sigaction (int mysignal)
 #ifdef SIGTERM
   if (mysignal == SIGTERM)
     {
+      strncpy (sh_sig_msg, _("Terminated"), 40);
       sig_termfast          = 1;
       ++sig_urgent;
     }
@@ -603,9 +608,12 @@ void sh_unix_sigaction (int mysignal)
 }
 
 static
-void sh_unix_siginstall (void)
+void sh_unix_siginstall (int goDaemon)
 {
   struct sigaction act, act_fast, act2, oldact, ignact;
+#if defined (SH_WITH_SERVER)
+  (void) goDaemon;
+#endif
 
   SL_ENTER(_("sh_unix_siginstall"));
 
@@ -690,22 +698,22 @@ void sh_unix_siginstall (void)
   /* signals to control the daemon */
 
 #ifdef SIGHUP
-  retry_sigaction(FIL__, __LINE__, SIGHUP,  &act2, &oldact);
+  retry_sigaction(FIL__, __LINE__, SIGHUP,     &act2, &oldact);
 #endif
 #ifdef SIGABRT
-  retry_sigaction(FIL__, __LINE__, SIGABRT, &act2, &oldact);
+  retry_sigaction(FIL__, __LINE__, SIGABRT,    &act2, &oldact);
 #endif
 #ifdef SIGUSR1
-  retry_sigaction(FIL__, __LINE__, SIGUSR1, &act2, &oldact);
+  retry_sigaction(FIL__, __LINE__, SIGUSR1,    &act2, &oldact);
 #endif
 #ifdef SIGUSR2
-  retry_sigaction(FIL__, __LINE__, SIGUSR2, &act2, &oldact);
+  retry_sigaction(FIL__, __LINE__, SIGUSR2,    &act2, &oldact);
 #endif
 #ifdef SIGQUIT
-  retry_sigaction(FIL__, __LINE__, SIGQUIT, &act2, &oldact);
+  retry_sigaction(FIL__, __LINE__, SIGQUIT,    &act2, &oldact);
 #endif
 #ifdef SIGTERM
-  retry_sigaction(FIL__, __LINE__, SIGTERM, &act,  &oldact);
+  retry_sigaction(FIL__, __LINE__, SIGTERM,     &act,  &oldact);
 #endif
 
   /* fatal signals that may cause termination */
@@ -728,63 +736,79 @@ void sh_unix_siginstall (void)
   /* other signals  */
 
 #ifdef SIGINT
-  retry_sigaction(FIL__, __LINE__, SIGINT,  &act, &oldact);
+  retry_sigaction(FIL__, __LINE__, SIGINT,       &act, &oldact);
 #endif
 #ifdef SIGPIPE
-  retry_sigaction(FIL__, __LINE__, SIGPIPE, &act, &oldact);
+  retry_sigaction(FIL__, __LINE__, SIGPIPE,      &act, &oldact);
 #endif
 #ifdef SIGALRM
-  retry_sigaction(FIL__, __LINE__, SIGALRM, &ignact, &oldact);
+  retry_sigaction(FIL__, __LINE__, SIGALRM,   &ignact, &oldact);
 #endif
 #ifdef SIGTSTP
-  retry_sigaction(FIL__, __LINE__, SIGTSTP, &ignact, &oldact);
+  retry_sigaction(FIL__, __LINE__, SIGTSTP,   &ignact, &oldact);
 #endif
 #ifdef SIGTTIN
-  retry_sigaction(FIL__, __LINE__, SIGTTIN, &act, &oldact);
+  retry_sigaction(FIL__, __LINE__, SIGTTIN,   &ignact, &oldact);
 #endif
+#if defined (SH_WITH_CLIENT) || defined (SH_STANDALONE)
 #ifdef SIGTTOU
-  retry_sigaction(FIL__, __LINE__, SIGTTOU, &ignact, &oldact);
+  if (goDaemon == 1)
+    retry_sigaction(FIL__, __LINE__, SIGTTOU,     &act2, &oldact);
+  else
+    retry_sigaction(FIL__, __LINE__, SIGTTOU,   &ignact, &oldact);
+#endif
+#else
+#ifdef SIGTTOU
+  retry_sigaction(FIL__, __LINE__, SIGTTOU,   &ignact, &oldact);
+#endif
 #endif
 
 #ifdef SIGTRAP
 #if !defined(SCREW_IT_UP)
-  retry_sigaction(FIL__, __LINE__, SIGTRAP, &act, &oldact);
+  retry_sigaction(FIL__, __LINE__, SIGTRAP,      &act, &oldact);
 #endif
 #endif
 
-#ifdef SIGEMT
-  retry_sigaction(FIL__, __LINE__, SIGEMT,  &ignact, &oldact);
-#endif
-#ifdef SIGSTKFLT
-  retry_sigaction(FIL__, __LINE__, SIGSTKFLT,&act, &oldact);
-#endif
-#ifdef SIGIO
-  retry_sigaction(FIL__, __LINE__, SIGIO,   &ignact, &oldact);
-#endif
 #ifdef SIGPOLL
-  retry_sigaction(FIL__, __LINE__, SIGPOLL, &ignact, &oldact);
+  retry_sigaction(FIL__, __LINE__, SIGPOLL,   &ignact, &oldact);
 #endif
-#ifdef SIGXCPU
-  retry_sigaction(FIL__, __LINE__, SIGXCPU, &act, &oldact);
+#if defined(SIGPROF) && !defined(SH_PROFILE)
+  retry_sigaction(FIL__, __LINE__, SIGPROF,   &ignact, &oldact);
 #endif
-#ifdef SIGXFSZ
-  retry_sigaction(FIL__, __LINE__, SIGXFSZ, &act, &oldact);
+#ifdef SIGSYS
+  retry_sigaction(FIL__, __LINE__, SIGSYS,       &act, &oldact);
+#endif
+#ifdef SIGURG
+  retry_sigaction(FIL__, __LINE__, SIGURG,    &ignact, &oldact);
 #endif
 #if defined(SIGVTALRM) && !defined(SH_PROFILE)
   retry_sigaction(FIL__, __LINE__, SIGVTALRM, &ignact, &oldact);
 #endif
-#if defined(SIGPROF) && !defined(SH_PROFILE)
-  retry_sigaction(FIL__, __LINE__, SIGPROF, &ignact, &oldact);
+#ifdef SIGXCPU
+  retry_sigaction(FIL__, __LINE__, SIGXCPU,      &act, &oldact);
+#endif
+#ifdef SIGXFSZ
+  retry_sigaction(FIL__, __LINE__, SIGXFSZ,      &act, &oldact);
+#endif
+
+#ifdef SIGEMT
+  retry_sigaction(FIL__, __LINE__, SIGEMT,    &ignact, &oldact);
+#endif
+#ifdef SIGSTKFLT
+  retry_sigaction(FIL__, __LINE__, SIGSTKFLT,    &act, &oldact);
+#endif
+#ifdef SIGIO
+  retry_sigaction(FIL__, __LINE__, SIGIO,     &ignact, &oldact);
 #endif
 #ifdef SIGPWR
-  retry_sigaction(FIL__, __LINE__, SIGPWR,  &act, &oldact);
+  retry_sigaction(FIL__, __LINE__, SIGPWR,       &act, &oldact);
 #endif
 
 #ifdef SIGLOST
-  retry_sigaction(FIL__, __LINE__, SIGLOST,  &ignact, &oldact);
+  retry_sigaction(FIL__, __LINE__, SIGLOST,   &ignact, &oldact);
 #endif
 #ifdef SIGUNUSED
-  retry_sigaction(FIL__, __LINE__, SIGUNUSED,  &ignact, &oldact);
+  retry_sigaction(FIL__, __LINE__, SIGUNUSED, &ignact, &oldact);
 #endif
 
   SL_RET0(_("sh_unix_siginstall"));
@@ -902,7 +926,7 @@ int tf_add_trusted_user(char * c)
   
   SL_ENTER(_("tf_add_trusted_user"));
 
-  q = strtok(p, ", ");
+  q = strtok(p, ", \t");
   if (!q)
     {
       SH_FREE(p);
@@ -916,7 +940,7 @@ int tf_add_trusted_user(char * c)
 	  SH_FREE(p);
 	  SL_RETURN((i), _("tf_add_trusted_user"));
 	}
-      q = strtok(NULL, ", ");
+      q = strtok(NULL, ", \t");
     }
   SH_FREE(p);
   SL_RETURN((0), _("tf_add_trusted_user"));
@@ -1044,7 +1068,11 @@ int tf_trust_check (char * file, int mode)
 #endif
 
 #ifdef HAVE_INITGROUPS
+#ifdef HOST_IS_OSF
+int  sh_unix_initgroups (      char * in_user, gid_t in_gid)
+#else
 int  sh_unix_initgroups (const char * in_user, gid_t in_gid)
+#endif
 {
   int status  = -1;
   status = sh_initgroups (in_user, in_gid);
@@ -1061,8 +1089,8 @@ int  sh_unix_initgroups (const char * in_user, gid_t in_gid)
 #else
 int  sh_unix_initgroups (const char * in_user, gid_t in_gid)
 {
-  const char * dummy = in_user;
-  gid_t dummy2 = in_gid;
+  (void) in_user;
+  (void) in_gid;
   return 0;
 }
 #endif
@@ -1092,8 +1120,8 @@ int  sh_unix_initgroups2 (uid_t in_pid, gid_t in_gid)
 #else
 int  sh_unix_initgroups2 (uid_t in_pid, gid_t in_gid)
 {
-  uid_t dummy  = in_user;
-  gid_t dummy2 = in_gid;
+  (void) in_pid;
+  (void) in_gid;
   return 0;
 }
 #endif
@@ -1101,10 +1129,14 @@ int  sh_unix_initgroups2 (uid_t in_pid, gid_t in_gid)
 void sh_unix_closeall (int fd, int except)
 {
   int fdx = fd;
+#ifdef _SC_OPEN_MAX
+  int fdlimit = sysconf (_SC_OPEN_MAX);
+#else
 #ifdef OPEN_MAX
   int fdlimit = OPEN_MAX;
 #else
-  int fdlimit = sysconf(_SC_OPEN_MAX);
+  int fdlimit = _POSIX_OPEN_MAX;
+#endif
 #endif
 
   SL_ENTER(_("sh_unix_closeall"));
@@ -1575,6 +1607,9 @@ int sh_unix_setdeamon(char * dummy)
 	   
   SL_RETURN(res, _("sh_unix_setdeamon"));
 }
+#if defined(HAVE_LIBPRELUDE)
+#include "sh_prelude.h"
+#endif
 
 int sh_unix_setnodeamon(char * dummy)
 {
@@ -1663,7 +1698,7 @@ int sh_unix_init(int goDaemon)
 #if defined(SCREW_IT_UP)
   sh_sigtrap_prepare();
 #endif
-  sh_unix_siginstall  ();
+  sh_unix_siginstall  (goDaemon);
 
   /* set file creation mask 
    */
@@ -1684,10 +1719,13 @@ int sh_unix_init(int goDaemon)
    */
   if (goDaemon == 1)
     {
-      sh_unix_closeall (0, -1);
+      sh_unix_closeall (0, -1);  /* if running as daemon */
       aud_open(FIL__, __LINE__, SL_NOPRIV, _("/dev/null"), O_RDWR, 0);
       retry_aud_dup(FIL__, __LINE__, 0); 
       retry_aud_dup(FIL__, __LINE__, 0);
+#if defined(HAVE_LIBPRELUDE)
+      sh_prelude_reset ();
+#endif
 
       /* --- wait until parent has exited ---
        */
@@ -1717,7 +1755,10 @@ int sh_unix_init(int goDaemon)
     }
   else
     {
-      sh_unix_closeall(3, -1);
+      sh_unix_closeall(3, -1);  /* if not daemon */
+#if defined(HAVE_LIBPRELUDE)
+      sh_prelude_reset ();
+#endif
 #if defined(SH_WITH_SERVER) 
       sh_socket_open_int ();
 #endif
@@ -1802,12 +1843,8 @@ unsigned long sh_unix_longtime ()
 unsigned long sh_unix_notime ()
 {
   struct timeval  tv;
-  struct timezone tz;
 
-  tz.tz_minuteswest = 0;
-  tz.tz_dsttime     = 0;
-
-  gettimeofday (&tv, &tz);
+  gettimeofday (&tv, NULL);
 
   return ((unsigned long)(tv.tv_sec + tv.tv_usec * 10835 + getpid() + getppid()));
   
@@ -2214,7 +2251,6 @@ int sh_unix_getUser ()
 }
 
 
-
 int sh_unix_getline (SL_TICKET fd, char * line, int sizeofline)
 {
   register int  count;
@@ -2232,8 +2268,6 @@ int sh_unix_getline (SL_TICKET fd, char * line, int sizeofline)
 
     count = sl_read (fd, &c, 1);
 
-    /* fprintf (stderr, "%c", c); */
-
     /* end of file
      */
     if (count < 1) {
@@ -2601,6 +2635,8 @@ int sh_unix_set_io_limit (char * c)
  */
 extern int flag_err_debug;
 
+#include "sh_ignore.h"
+
 int sh_unix_getinfo (int level, char * filename, file_type * theFile, 
 		     char * fileHash, int policy)
 {
@@ -2713,17 +2749,19 @@ int sh_unix_getinfo (int level, char * filename, file_type * theFile,
 	  (sh_hash_have_it (theFile->fullpath) >= 0 && 
 	   theFile->reported == S_FALSE))
 	{
-	  tmp2 = sh_util_safe_name (theFile->fullpath);
-	  sh_error_handle (level, FIL__, __LINE__, stat_return, MSG_FI_LSTAT,
-			   sh_error_message (stat_return), tmp2);
-	  SH_FREE(tmp2);
+	  if (S_FALSE == sh_ignore_chk_del(theFile->fullpath)) {
+	    tmp2 = sh_util_safe_name (theFile->fullpath);
+	    sh_error_handle (level, FIL__, __LINE__, stat_return, MSG_FI_LSTAT,
+			     sh_error_message (stat_return), tmp2);
+	    SH_FREE(tmp2);
+	  }
 	}
       SL_RETURN((-1),_("sh_unix_getinfo"));
     }
 
-  /* ---  case 2: not a regular file  --- 
+  /* ---  case 2: not a regular file, or [IgnoreAll]  --- 
    */
-  else if (! S_ISREG(buf.st_mode) )
+  else if (! S_ISREG(buf.st_mode))
     {
       if (fileHash != NULL)
 	strcpy(fileHash,                              /* known to fit */
@@ -2741,8 +2779,14 @@ int sh_unix_getinfo (int level, char * filename, file_type * theFile,
     {
       if (fileHash != NULL)
 	{
-	  if (policy == SH_LEVEL_PRELINK && 
-	      S_TRUE == sh_prelink_iself(rval_open,fbuf.st_size,alert_timeout))
+	  if ((theFile->check_mask & MODI_CHK) == 0)
+	    {
+	      strcpy(fileHash,                         /* known to fit */
+		     _("000000000000000000000000000000000000000000000000"));
+	    }
+	  else if (policy == SH_LEVEL_PRELINK && 
+		   S_TRUE == sh_prelink_iself(rval_open, fbuf.st_size, 
+					      alert_timeout))
 	    {
 	      if (0 != sh_prelink_run (theFile->fullpath, 
 				       fileHash, alert_timeout))
@@ -2767,8 +2811,14 @@ int sh_unix_getinfo (int level, char * filename, file_type * theFile,
 
       if (fileHash != NULL)
 	{
-	  if (policy == SH_LEVEL_PRELINK &&
-              S_TRUE == sh_prelink_iself(rval_open,fbuf.st_size,alert_timeout))
+	  if ((theFile->check_mask & MODI_CHK) == 0)
+	    {
+	      strcpy(fileHash,                         /* known to fit */
+		     _("000000000000000000000000000000000000000000000000"));
+	    }
+	  else if (policy == SH_LEVEL_PRELINK &&
+		   S_TRUE == sh_prelink_iself(rval_open, fbuf.st_size, 
+					      alert_timeout))
 	    {
 	      if (0 != sh_prelink_run (theFile->fullpath, 
 				       fileHash, alert_timeout))
@@ -2988,7 +3038,8 @@ int sh_unix_getinfo (int level, char * filename, file_type * theFile,
 	tmp2 = sh_util_safe_name (theFile->linkpath);
 	if (stat_return != ENOENT)
 	  { 
-	    sh_error_handle (level, FIL__, __LINE__, stat_return, MSG_FI_LSTAT,
+	    sh_error_handle (level, FIL__, __LINE__, stat_return, 
+			     MSG_FI_LSTAT,
 			     sh_error_message (stat_return), tmp2);
 	  }
 	else 
@@ -3974,17 +4025,15 @@ volatile int sh_not_traced = 0;
 
 #ifdef HAVE_GETTIMEOFDAY
 struct timeval  save_tv;
-struct timezone save_tz;
 #endif
 
 void sh_sigtrap_handler (int signum)
 {
 #ifdef HAVE_GETTIMEOFDAY
   struct timeval  tv;
-  struct timezone tz;
   long   difftv;
   
-  gettimeofday(&tv, &tz);
+  gettimeofday(&tv, NULL);
   difftv = (tv.tv_sec - save_tv.tv_sec) * 1000000 + 
     (tv.tv_usec - save_tv.tv_usec);
   if (difftv > 500000)
