@@ -1306,7 +1306,7 @@ int sh_util_obscurename (ShErrLevel level, char * name_orig, int flag)
 	   (*name) == '\v' || iscntrl((int) *name) || 
 	   ((*name) != ' ' && !isgraph ((int) *name)) ) 
 	{
-	  i = (unsigned int) *name;
+	  i = (unsigned char) *name;
 	  if (sh_obscure_index[i] != (unsigned char)1)
 	    {
 	      if (flag == S_TRUE)
