@@ -12,7 +12,18 @@
 #include <sys/types.h>
 #include <fcntl.h>
 #include <signal.h>
+
+#if TIME_WITH_SYS_TIME
+#include <sys/time.h>
 #include <time.h>
+#else
+#if HAVE_SYS_TIME_H
+#include <sys/time.h>
+#else
+#include <time.h>
+#endif
+#endif
+
 #ifdef HAVE_MEMORY_H
 #include <memory.h>
 #endif
@@ -1890,13 +1901,13 @@ SL_TICKET sl_open_rdwr (char * fname, int priv)
 SL_TICKET sl_open_safe_rdwr (char * fname, int priv)
 {
   long status;
-  SL_ENTER(_("sl_open_rdwr"));
+  SL_ENTER(_("sl_open_safe_rdwr"));
 
   if (SL_ENONE != (status = check_fname_priv (fname, priv)))
-    SL_IRETURN(status, _("sl_open_rdwr"));
+    SL_IRETURN(status, _("sl_open_safe_rdwr"));
 
   status = sl_open_file(fname, SL_OPEN_SAFE_RDWR, priv);
-  SL_IRETURN(status, _("sl_open_rdwr"));
+  SL_IRETURN(status, _("sl_open_safe_rdwr"));
 }
 
 SL_TICKET sl_open_write_trunc (char * fname, int priv)
