@@ -211,8 +211,8 @@ uid_t rootonly[] = { SL_ALWAYS_TRUSTED,
 		    tf_uid_neg, tf_uid_neg, tf_uid_neg, tf_uid_neg };
 
 uid_t tf_euid = tf_uid_neg;
-int EUIDSLOT = sizeof(test_rootonly)/sizeof(long);
-int ORIG_EUIDSLOT = sizeof(test_rootonly)/sizeof(long);
+int EUIDSLOT = sizeof(test_rootonly)/sizeof(uid_t);
+int ORIG_EUIDSLOT = sizeof(test_rootonly)/sizeof(uid_t);
 
 char  tf_path[MAXFILENAME];		/* error path for trust function */
 uid_t tf_baduid;
